@@ -22,9 +22,9 @@ from oslo_log import log
 from six.moves import urllib
 
 from keystone.assignment import schema
+from keystone.common import authorization
 from keystone.common import controller
 from keystone.common import dependency
-from keystone.common import utils
 from keystone.common import validation
 from keystone.common import wsgi
 import keystone.conf
@@ -50,7 +50,7 @@ class TenantAssignment(controller.V2Controller):
         Doesn't care about token scopedness.
 
         """
-        token_ref = utils.get_token_ref(request.context_dict)
+        token_ref = authorization.get_token_ref(request.context_dict)
 
         tenant_refs = (
             self.assignment_api.list_projects_for_user(token_ref.user_id))
@@ -266,8 +266,7 @@ class ProjectAssignmentV3(controller.V3Controller):
     @controller.filterprotected('domain_id', 'enabled', 'name')
     def list_user_projects(self, request, filters, user_id):
         hints = ProjectAssignmentV3.build_driver_hints(request, filters)
-        refs = self.assignment_api.list_projects_for_user(user_id,
-                                                          hints=hints)
+        refs = self.assignment_api.list_projects_for_user(user_id)
         return ProjectAssignmentV3.wrap_collection(request.context_dict,
                                                    refs,
                                                    hints=hints)
@@ -500,6 +499,10 @@ class ImpliedRolesV3(controller.V3Controller):
         implied_role = self.role_api.get_role(implied_id)
         stanza = self._implied_role_stanza(endpoint, implied_role)
         response["role_inference"]['implies'] = stanza
+        response["links"] = {
+            "self": (endpoint + "/v3/roles/" + prior_id
+                     + "/implies/" + implied_id)
+        }
         return response
 
     @controller.protected(callback=_check_implies_role)
@@ -964,7 +967,7 @@ class RoleAssignmentV3(controller.V3Controller):
 
         """
         ref = {}
-        for filter, value in protection_info['filter_attr'].items():
+        for filter, value in protection_info.get('filter_attr', {}).items():
             if filter == 'scope.project.id' and value:
                 ref['project'] = self.resource_api.get_project(value)
 
