@@ -12,5 +12,8 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+# NOTE(notmorgan): Be careful in adjusting whitespace here, flake8 checks
+# get cranky.
+from keystone.auth import core  # noqa
+
 from keystone.auth import controllers  # noqa
-from keystone.auth.core import *  # noqa
