@@ -19,7 +19,9 @@ import os
 import sys
 import uuid
 
+import migrate
 from oslo_config import cfg
+from oslo_db.sqlalchemy import migration
 from oslo_log import log
 from oslo_log import versionutils
 from oslo_serialization import jsonutils
@@ -28,7 +30,6 @@ import pbr.version
 from keystone.cmd import doctor
 from keystone.common import driver_hints
 from keystone.common import fernet_utils
-from keystone.common import openssl
 from keystone.common import sql
 from keystone.common.sql import upgrades
 from keystone.common import utils
@@ -37,7 +38,7 @@ from keystone.credential.providers import fernet as credential_fernet
 from keystone import exception
 from keystone.federation import idp
 from keystone.federation import utils as mapping_engine
-from keystone.i18n import _, _LE, _LI, _LW
+from keystone.i18n import _
 from keystone.server import backends
 from keystone import token
 
@@ -107,17 +108,17 @@ class BootStrap(BaseApp):
                             metavar='OS_BOOTSTRAP_ADMIN_URL',
                             help=('The initial identity admin url created '
                                   'during the keystone bootstrap process. '
-                                  'e.g. http://127.0.0.1:35357/v2.0'))
+                                  'e.g. http://127.0.0.1:35357/v3'))
         parser.add_argument('--bootstrap-public-url',
                             metavar='OS_BOOTSTRAP_PUBLIC_URL',
                             help=('The initial identity public url created '
                                   'during the keystone bootstrap process. '
-                                  'e.g. http://127.0.0.1:5000/v2.0'))
+                                  'e.g. http://127.0.0.1:5000/v3'))
         parser.add_argument('--bootstrap-internal-url',
                             metavar='OS_BOOTSTRAP_INTERNAL_URL',
                             help=('The initial identity internal url created '
                                   'during the keystone bootstrap process. '
-                                  'e.g. http://127.0.0.1:5000/v2.0'))
+                                  'e.g. http://127.0.0.1:5000/v3'))
         parser.add_argument('--bootstrap-region-id',
                             metavar='OS_BOOTSTRAP_REGION_ID',
                             help=('The initial region_id endpoints will be '
@@ -188,10 +189,10 @@ class BootStrap(BaseApp):
             self.resource_manager.create_domain(
                 domain_id=default_domain['id'],
                 domain=default_domain)
-            LOG.info(_LI('Created domain %s'), default_domain['id'])
+            LOG.info('Created domain %s', default_domain['id'])
         except exception.Conflict:
             # NOTE(morganfainberg): Domain already exists, continue on.
-            LOG.info(_LI('Domain %s already exists, skipping creation.'),
+            LOG.info('Domain %s already exists, skipping creation.',
                      default_domain['id'])
 
         try:
@@ -204,9 +205,9 @@ class BootStrap(BaseApp):
                                         'the cloud.',
                          'name': self.project_name}
             )
-            LOG.info(_LI('Created project %s'), self.project_name)
+            LOG.info('Created project %s', self.project_name)
         except exception.Conflict:
-            LOG.info(_LI('Project %s already exists, skipping creation.'),
+            LOG.info('Project %s already exists, skipping creation.',
                      self.project_name)
             project = self.resource_manager.get_project_by_name(
                 self.project_name, default_domain['id'])
@@ -216,7 +217,7 @@ class BootStrap(BaseApp):
         try:
             user = self.identity_manager.get_user_by_name(self.username,
                                                           default_domain['id'])
-            LOG.info(_LI('User %s already exists, skipping creation.'),
+            LOG.info('User %s already exists, skipping creation.',
                      self.username)
 
             # If the user is not enabled, re-enable them. This also helps
@@ -244,12 +245,12 @@ class BootStrap(BaseApp):
             # as a recovery tool, without having to create a new user.
             if update:
                 user = self.identity_manager.update_user(user['id'], update)
-                LOG.info(_LI('Reset password for user %s.'), self.username)
+                LOG.info('Reset password for user %s.', self.username)
                 if not enabled and user['enabled']:
                     # Although we always try to enable the user, this log
                     # message only makes sense if we know that the user was
                     # previously disabled.
-                    LOG.info(_LI('Enabled user %s.'), self.username)
+                    LOG.info('Enabled user %s.', self.username)
         except exception.UserNotFound:
             user = self.identity_manager.create_user(
                 user_ref={'name': self.username,
@@ -258,7 +259,7 @@ class BootStrap(BaseApp):
                           'password': self.password
                           }
             )
-            LOG.info(_LI('Created user %s'), self.username)
+            LOG.info('Created user %s', self.username)
 
         # NOTE(morganfainberg): Do not create the role if it already exists.
         try:
@@ -267,9 +268,9 @@ class BootStrap(BaseApp):
                 role={'name': self.role_name,
                       'id': self.role_id},
             )
-            LOG.info(_LI('Created role %s'), self.role_name)
+            LOG.info('Created role %s', self.role_name)
         except exception.Conflict:
-            LOG.info(_LI('Role %s exists, skipping creation.'), self.role_name)
+            LOG.info('Role %s exists, skipping creation.', self.role_name)
             # NOTE(davechen): There is no backend method to get the role
             # by name, so build the hints to list the roles and filter by
             # name instead.
@@ -286,14 +287,14 @@ class BootStrap(BaseApp):
                 tenant_id=self.project_id,
                 role_id=self.role_id
             )
-            LOG.info(_LI('Granted %(role)s on %(project)s to user'
-                         ' %(username)s.'),
+            LOG.info('Granted %(role)s on %(project)s to user'
+                     ' %(username)s.',
                      {'role': self.role_name,
                       'project': self.project_name,
                       'username': self.username})
         except exception.Conflict:
-            LOG.info(_LI('User %(username)s already has %(role)s on '
-                         '%(project)s.'),
+            LOG.info('User %(username)s already has %(role)s on '
+                     '%(project)s.',
                      {'username': self.username,
                       'role': self.role_name,
                       'project': self.project_name})
@@ -303,9 +304,9 @@ class BootStrap(BaseApp):
                 self.catalog_manager.create_region(
                     region_ref={'id': self.region_id}
                 )
-                LOG.info(_LI('Created region %s'), self.region_id)
+                LOG.info('Created region %s', self.region_id)
             except exception.Conflict:
-                LOG.info(_LI('Region %s exists, skipping creation.'),
+                LOG.info('Region %s exists, skipping creation.',
                          self.region_id)
 
         if self.public_url or self.admin_url or self.internal_url:
@@ -362,12 +363,12 @@ class BootStrap(BaseApp):
                         endpoint_id=endpoint_ref['id'],
                         endpoint_ref=endpoint_ref)
 
-                    LOG.info(_LI('Created %(interface)s endpoint %(url)s'),
+                    LOG.info('Created %(interface)s endpoint %(url)s',
                              {'interface': interface, 'url': url})
                 else:
                     # NOTE(jamielennox): electing not to update existing
                     # endpoints here. There may be call to do so in future.
-                    LOG.info(_LI('Skipping %s endpoint as already created'),
+                    LOG.info('Skipping %s endpoint as already created',
                              interface)
 
                 self.endpoints[interface] = endpoint_ref['id']
@@ -445,15 +446,78 @@ class DbSync(BaseApp):
                                  'that are no longer required. This command '
                                  'should be run after all keystone nodes are '
                                  'running the new release.'))
+
+        group.add_argument('--check', default=False, action='store_true',
+                           help=('Check for outstanding database actions that '
+                                 'still need to be executed. This command can '
+                                 'be used to verify the condition of the '
+                                 'current database state.'))
         return parser
 
+    @classmethod
+    def check_db_sync_status(self):
+        status = 0
+        try:
+            expand_version = upgrades.get_db_version(repo='expand_repo')
+        except migration.exception.DbMigrationError:
+            LOG.info('Your database is not currently under version '
+                     'control or the database is already controlled. Your '
+                     'first step is to run `keystone-manage db_sync '
+                     '--expand`.')
+            return 2
+        try:
+            migrate_version = upgrades.get_db_version(
+                repo='data_migration_repo')
+        except migration.exception.DbMigrationError:
+            migrate_version = 0
+        try:
+            contract_version = upgrades.get_db_version(repo='contract_repo')
+        except migration.exception.DbMigrationError:
+            contract_version = 0
+
+        repo = migrate.versioning.repository.Repository(
+            upgrades.find_repo('expand_repo'))
+        migration_script_version = int(max(repo.versions.versions))
+
+        if (contract_version > migrate_version or migrate_version >
+                expand_version):
+            LOG.info('Your database is out of sync. For more information '
+                     'refer to https://docs.openstack.org/developer/'
+                     'keystone/upgrading.html')
+            status = 1
+        elif migration_script_version > expand_version:
+            LOG.info('Your database is not up to date. Your first step is '
+                     'to run `keystone-manage db_sync --expand`.')
+            status = 2
+        elif expand_version > migrate_version:
+            LOG.info('Expand version is ahead of migrate. Your next step '
+                     'is to run `keystone-manage db_sync --migrate`.')
+            status = 3
+        elif migrate_version > contract_version:
+            LOG.info('Migrate version is ahead of contract. Your next '
+                     'step is to run `keystone-manage db_sync --contract`.')
+            status = 4
+        elif (migration_script_version == expand_version == migrate_version ==
+                contract_version):
+            LOG.info('All db_sync commands are upgraded to the same '
+                     'version and up-to-date.')
+        LOG.info('The latest installed migration script version is: '
+                 '%(script)d.\nCurrent repository versions:\nExpand: '
+                 '%(expand)d \nMigrate: %(migrate)d\nContract: '
+                 '%(contract)d', {'script': migration_script_version,
+                                  'expand': expand_version,
+                                  'migrate': migrate_version,
+                                  'contract': contract_version})
+        return status
+
     @staticmethod
     def main():
         assert_not_extension(CONF.command.extension)
-
         # It is possible to run expand and migrate at the same time,
         # expand needs to run first however.
-        if CONF.command.expand and CONF.command.migrate:
+        if CONF.command.check:
+            sys.exit(DbSync.check_db_sync_status())
+        elif CONF.command.expand and CONF.command.migrate:
             upgrades.expand_schema()
             upgrades.migrate_data()
         elif CONF.command.expand:
@@ -523,44 +587,6 @@ class BasePermissionsSetup(BaseApp):
         return keystone_user_id, keystone_group_id
 
 
-class BaseCertificateSetup(BasePermissionsSetup):
-    """Provides common options for certificate setup."""
-
-    @classmethod
-    def add_argument_parser(cls, subparsers):
-        parser = super(BaseCertificateSetup,
-                       cls).add_argument_parser(subparsers)
-        parser.add_argument('--rebuild', default=False, action='store_true',
-                            help=('Rebuild certificate files: erase previous '
-                                  'files and regenerate them.'))
-        return parser
-
-
-class PKISetup(BaseCertificateSetup):
-    """Setup keys and certificates for signing and verifying revocation lists.
-
-    This is NOT intended for production use, see Keystone Configuration
-    documentation for details. As of the Mitaka release, this command has
-    been DEPRECATED and may be removed in the 'O' release.
-    """
-
-    name = 'pki_setup'
-
-    @classmethod
-    def main(cls):
-        versionutils.report_deprecated_feature(
-            LOG,
-            _LW("keystone-manage pki_setup is deprecated as of Mitaka in "
-                "favor of not using PKI tokens and may be removed in 'O' "
-                "release."))
-        LOG.warning(_LW('keystone-manage pki_setup is not recommended for '
-                        'production use.'))
-        keystone_user_id, keystone_group_id = cls.get_user_group()
-        conf_pki = openssl.ConfigurePKI(keystone_user_id, keystone_group_id,
-                                        rebuild=CONF.command.rebuild)
-        conf_pki.run()
-
-
 class FernetSetup(BasePermissionsSetup):
     """Setup a key repository for Fernet tokens.
 
@@ -788,8 +814,8 @@ class TokenFlush(BaseApp):
         except exception.NotImplemented:
             # NOTE(ravelar159): Stop NotImplemented from unsupported token
             # driver when using token_flush and print out warning instead
-            LOG.warning(_LW('Token driver %s does not support token_flush. '
-                            'The token_flush command had no effect.'),
+            LOG.warning('Token driver %s does not support token_flush. '
+                        'The token_flush command had no effect.',
                         CONF.token.driver)
 
 
@@ -885,7 +911,7 @@ def _domain_config_finder(conf_dir):
 
     :returns: generator yielding (filename, domain_name) tuples
     """
-    LOG.info(_LI('Scanning %r for domain config files'), conf_dir)
+    LOG.info('Scanning %r for domain config files', conf_dir)
     for r, d, f in os.walk(conf_dir):
         for fname in f:
             if (fname.startswith(DOMAIN_CONF_FHEAD) and
@@ -896,8 +922,8 @@ def _domain_config_finder(conf_dir):
                     yield (os.path.join(r, fname), domain_name)
                     continue
 
-            LOG.warning(_LW('Ignoring file (%s) while scanning '
-                            'domain config directory'), fname)
+            LOG.warning('Ignoring file (%s) while scanning '
+                        'domain config directory', fname)
 
 
 class DomainConfigUploadFiles(object):
@@ -985,8 +1011,8 @@ class DomainConfigUploadFiles(object):
                                                      sections)
             return True
         except Exception as e:
-            msg = _LE('Error processing config file for domain: '
-                      '%(domain_name)s, file: %(filename)s, error: %(error)s')
+            msg = ('Error processing config file for domain: '
+                   '%(domain_name)s, file: %(filename)s, error: %(error)s')
             LOG.error(msg,
                       {'domain_name': domain_name,
                        'filename': file_name,
@@ -1022,13 +1048,13 @@ class DomainConfigUploadFiles(object):
         for filename, domain_name in self._domain_config_finder(conf_dir):
             if self._upload_config_to_database(filename, domain_name):
                 success_cnt += 1
-                LOG.info(_LI('Successfully uploaded domain config %r'),
+                LOG.info('Successfully uploaded domain config %r',
                          filename)
             else:
                 failure_cnt += 1
 
         if success_cnt == 0:
-            LOG.warning(_LW('No domain configs uploaded from %r'), conf_dir)
+            LOG.warning('No domain configs uploaded from %r', conf_dir)
 
         if failure_cnt:
             return False
@@ -1076,9 +1102,9 @@ class DomainConfigUpload(BaseApp):
     def main():
         versionutils.report_deprecated_feature(
             LOG,
-            _LW("keystone-manage domain_config_upload is deprecated as of "
-                "Newton in favor of setting domain config options via the API "
-                "and may be removed in 'P' release."))
+            "keystone-manage domain_config_upload is deprecated as of "
+            "Newton in favor of setting domain config options via the API "
+            "and may be removed in 'P' release.")
         dcu = DomainConfigUploadFiles()
         status = dcu.run()
         if status is not None:
@@ -1277,7 +1303,6 @@ CMDS = [
     MappingPopulate,
     MappingPurge,
     MappingEngineTester,
-    PKISetup,
     SamlIdentityProviderMetadata,
     TokenFlush,
 ]
@@ -1307,6 +1332,6 @@ def main(argv=None, config_files=None):
          usage='%(prog)s [' + '|'.join([cmd.name for cmd in CMDS]) + ']',
          default_config_files=config_files)
     if not CONF.default_config_files:
-        LOG.warning(_LW('Config file not found, using default configs.'))
+        LOG.warning('Config file not found, using default configs.')
     keystone.conf.setup_logging()
     CONF.command.cmd_class.main()
