@@ -18,15 +18,19 @@ Before using this module, call initialize(). This has to be done before
 CONF() because it sets up configuration options.
 
 """
+import datetime
 import functools
+import pytz
 
-from oslo_config import cfg
 from oslo_db import exception as db_exception
 from oslo_db import options as db_options
 from oslo_db.sqlalchemy import enginefacade
 from oslo_db.sqlalchemy import models
 from oslo_log import log
 from oslo_serialization import jsonutils
+from oslo_utils import timeutils
+from osprofiler import opts as profiler
+import osprofiler.sqlalchemy
 import six
 import sqlalchemy as sql
 from sqlalchemy.ext import declarative
@@ -35,11 +39,12 @@ from sqlalchemy import types as sql_types
 
 from keystone.common import driver_hints
 from keystone.common import utils
+import keystone.conf
 from keystone import exception
 from keystone.i18n import _
 
 
-CONF = cfg.CONF
+CONF = keystone.conf.CONF
 LOG = log.getLogger(__name__)
 
 ModelBase = declarative.declarative_base()
@@ -53,6 +58,8 @@ Integer = sql.Integer
 Enum = sql.Enum
 ForeignKey = sql.ForeignKey
 DateTime = sql.DateTime
+Date = sql.Date
+TIMESTAMP = sql.TIMESTAMP
 IntegrityError = sql.exc.IntegrityError
 DBDuplicateEntry = db_exception.DBDuplicateEntry
 OperationalError = sql.exc.OperationalError
@@ -71,6 +78,8 @@ def initialize():
     db_options.set_defaults(
         CONF,
         connection="sqlite:///keystone.db")
+    # Configure OSprofiler options
+    profiler.set_defaults(CONF, enabled=False, trace_sqlalchemy=False)
 
 
 def initialize_decorator(init):
@@ -117,26 +126,94 @@ class JsonBlob(sql_types.TypeDecorator):
         return jsonutils.loads(value)
 
 
-class DictBase(models.ModelBase):
+class DateTimeInt(sql_types.TypeDecorator):
+    """A column that automatically converts a datetime object to an Int.
+
+    Keystone relies on accurate (sub-second) datetime objects. In some cases
+    the RDBMS drop sub-second accuracy (some versions of MySQL). This field
+    automatically converts the value to an INT when storing the data and
+    back to a datetime object when it is loaded from the database.
+
+    NOTE: Any datetime object that has timezone data will be converted to UTC.
+          Any datetime object that has no timezone data will be assumed to be
+          UTC and loaded from the DB as such.
+    """
+
+    impl = sql.BigInteger
+    epoch = datetime.datetime.fromtimestamp(0, tz=pytz.UTC)
+
+    def process_bind_param(self, value, dialect):
+        if value is None:
+            return value
+        else:
+            if not isinstance(value, datetime.datetime):
+                raise ValueError(_('Programming Error: value to be stored '
+                                   'must be a datetime object.'))
+            value = timeutils.normalize_time(value)
+            value = value.replace(tzinfo=pytz.UTC)
+            # NOTE(morgan): We are casting this to an int, and ensuring we
+            # preserve microsecond data by moving the decimal. This is easier
+            # than being concerned with the differences in Numeric types in
+            # different SQL backends.
+            return int((value - self.epoch).total_seconds() * 1000000)
+
+    def process_result_value(self, value, dialect):
+        if value is None:
+            return None
+        else:
+            # Convert from INT to appropriate micro-second float (microseconds
+            # after the decimal) from what was stored to the DB
+            value = float(value) / 1000000
+            # NOTE(morgan): Explictly use timezone "pytz.UTC" to ensure we are
+            # not adjusting the actual datetime object from what we stored.
+            dt_obj = datetime.datetime.fromtimestamp(value, tz=pytz.UTC)
+            # Return non-tz aware datetime object (as keystone expects)
+            return timeutils.normalize_time(dt_obj)
+
+
+class ModelDictMixinWithExtras(models.ModelBase):
+    """Mixin making model behave with dict-like interfaces includes extras.
+
+    NOTE: DO NOT USE THIS FOR FUTURE SQL MODELS. "Extra" column is a legacy
+          concept that should not be carried forward with new SQL models
+          as the concept of "arbitrary" properties is not in line with
+          the design philosophy of Keystone.
+    """
+
     attributes = []
+    _msg = ('Programming Error: Model does not have an "extra" column. '
+            'Unless the model already has an "extra" column and has '
+            'existed in a previous released version of keystone with '
+            'the extra column included, the model should use '
+            '"ModelDictMixin" instead.')
 
     @classmethod
     def from_dict(cls, d):
         new_d = d.copy()
 
-        new_d['extra'] = {k: new_d.pop(k) for k in six.iterkeys(d)
+        if not hasattr(cls, 'extra'):
+            # NOTE(notmorgan): No translation here, This is an error for
+            # programmers NOT end users.
+            raise AttributeError(cls._msg)  # no qa
+
+        new_d['extra'] = {k: new_d.pop(k) for k in d.keys()
                           if k not in cls.attributes and k != 'extra'}
 
         return cls(**new_d)
 
     def to_dict(self, include_extra_dict=False):
-        """Returns the model's attributes as a dictionary.
+        """Return the model's attributes as a dictionary.
 
         If include_extra_dict is True, 'extra' attributes are literally
         included in the resulting dictionary twice, for backwards-compatibility
         with a broken implementation.
 
         """
+        if not hasattr(self, 'extra'):
+            # NOTE(notmorgan): No translation here, This is an error for
+            # programmers NOT end users.
+            raise AttributeError(self._msg)  # no qa
+
         d = self.extra.copy()
         for attr in self.__class__.attributes:
             d[attr] = getattr(self, attr)
@@ -147,20 +224,21 @@ class DictBase(models.ModelBase):
         return d
 
     def __getitem__(self, key):
+        """Evaluate if key is in extra or not, to return correct item."""
         if key in self.extra:
             return self.extra[key]
         return getattr(self, key)
 
 
-class ModelDictMixin(object):
+class ModelDictMixin(models.ModelBase):
 
     @classmethod
     def from_dict(cls, d):
-        """Returns a model instance from a dictionary."""
+        """Return a model instance from a dictionary."""
         return cls(**d)
 
     def to_dict(self):
-        """Returns the model's attributes as a dictionary."""
+        """Return the model's attributes as a dictionary."""
         names = (column.name for column in self.__table__.columns)
         return {name: getattr(self, name) for name in names}
 
@@ -196,12 +274,32 @@ def _get_context():
     return _CONTEXT
 
 
+# Unit tests set this to True so that oslo.db's global engine is used.
+# This allows oslo_db.test_base.DbTestCase to override the transaction manager
+# with its test transaction manager.
+_TESTING_USE_GLOBAL_CONTEXT_MANAGER = False
+
+
 def session_for_read():
-    return _get_main_context_manager().reader.using(_get_context())
+    if _TESTING_USE_GLOBAL_CONTEXT_MANAGER:
+        reader = enginefacade.reader
+    else:
+        reader = _get_main_context_manager().reader
+    return _wrap_session(reader.using(_get_context()))
 
 
 def session_for_write():
-    return _get_main_context_manager().writer.using(_get_context())
+    if _TESTING_USE_GLOBAL_CONTEXT_MANAGER:
+        writer = enginefacade.writer
+    else:
+        writer = _get_main_context_manager().writer
+    return _wrap_session(writer.using(_get_context()))
+
+
+def _wrap_session(sess):
+    if CONF.profiler.enabled and CONF.profiler.trace_sqlalchemy:
+        sess = osprofiler.sqlalchemy.wrap_session(sql, sess)
+    return sess
 
 
 def truncated(f):
@@ -242,7 +340,7 @@ class _WontMatch(Exception):
 
 
 def _filter(model, query, hints):
-    """Applies filtering to a query.
+    """Apply filtering to a query.
 
     :param model: the table model in question
     :param query: query to apply filters to
@@ -250,11 +348,11 @@ def _filter(model, query, hints):
                   Any filters satisfied here will be removed so that
                   the caller will know if any filters remain.
 
-    :returns query: query, updated with any filters satisfied
+    :returns: query updated with any filters satisfied
 
     """
     def inexact_filter(model, query, filter_, satisfied_filters):
-        """Applies an inexact filter to a query.
+        """Apply an inexact filter to a query.
 
         :param model: the table model in question
         :param query: query to apply filters to
@@ -262,8 +360,7 @@ def _filter(model, query, hints):
         :param list satisfied_filters: filter_ will be added if it is
                                        satisfied.
 
-        :returns query: query updated to add any inexact filters we could
-                        satisfy
+        :returns: query updated to add any inexact filters satisfied
 
         """
         column_attr = getattr(model, filter_['name'])
@@ -295,15 +392,14 @@ def _filter(model, query, hints):
         return query.filter(query_term)
 
     def exact_filter(model, query, filter_, satisfied_filters):
-        """Applies an exact filter to a query.
+        """Apply an exact filter to a query.
 
         :param model: the table model in question
         :param query: query to apply filters to
         :param dict filter_: describes this filter
         :param list satisfied_filters: filter_ will be added if it is
                                        satisfied.
-        :returns query: query updated to add any exact filters we could
-                        satisfy
+        :returns: query updated to add any exact filters satisfied
         """
         key = filter_['name']
 
@@ -340,12 +436,12 @@ def _filter(model, query, hints):
 
 
 def _limit(query, hints):
-    """Applies a limit to a query.
+    """Apply a limit to a query.
 
     :param query: query to apply filters to
     :param hints: contains the list of filters and limit details.
 
-    :returns: updated query
+    :returns: query updated with any limits satisfied
 
     """
     # NOTE(henry-nash): If we were to implement pagination, then we
@@ -362,7 +458,7 @@ def _limit(query, hints):
 
 
 def filter_limit_query(model, query, hints):
-    """Applies filtering and limit to a query.
+    """Apply filtering and limit to a query.
 
     :param model: table model
     :param query: query to apply filters to
@@ -372,7 +468,7 @@ def filter_limit_query(model, query, hints):
                   satisfied here will be removed so that the caller will
                   know if any filters remain.
 
-    :returns: updated query
+    :returns: query updated with any filters and limits satisfied
 
     """
     if hints is None:
@@ -399,7 +495,7 @@ def filter_limit_query(model, query, hints):
 
 
 def handle_conflicts(conflict_type='object'):
-    """Converts select sqlalchemy exceptions into HTTP 409 Conflict."""
+    """Convert select sqlalchemy exceptions into HTTP 409 Conflict."""
     _conflict_msg = 'Conflict %(conflict_type)s: %(details)s'
 
     def decorator(method):
@@ -413,8 +509,36 @@ def handle_conflicts(conflict_type='object'):
                 # as it can contain raw SQL.
                 LOG.debug(_conflict_msg, {'conflict_type': conflict_type,
                                           'details': six.text_type(e)})
+                name = None
+                field = None
+                domain_id = None
+                # First element is unnessecary for extracting name and causes
+                # object not iterable error. Remove it.
+                params = args[1:]
+                # We want to store the duplicate objects name in the error
+                # message for the user. If name is not available we use the id.
+                for arg in params:
+                    if isinstance(arg, dict):
+                        if 'name' in arg:
+                            field = 'name'
+                            name = arg['name']
+                        elif 'id' in arg:
+                            field = 'ID'
+                            name = arg['id']
+                        if 'domain_id' in arg:
+                            domain_id = arg['domain_id']
+                msg = _('Duplicate entry')
+                if name and domain_id:
+                    msg = _('Duplicate entry found with %(field)s %(name)s '
+                            'at domain ID %(domain_id)s') % {
+                        'field': field, 'name': name, 'domain_id': domain_id}
+                elif name:
+                    msg = _('Duplicate entry found with %(field)s '
+                            '%(name)s') % {'field': field, 'name': name}
+                elif domain_id:
+                    msg = (_('Duplicate entry at domain ID %s') % domain_id)
                 raise exception.Conflict(type=conflict_type,
-                                         details=_('Duplicate Entry'))
+                                         details=msg)
             except db_exception.DBError as e:
                 # TODO(blk-u): inspecting inner_exception breaks encapsulation;
                 # oslo_db should provide exception we need.
