@@ -353,7 +353,7 @@ class Auth(auth_controllers.Auth):
 
         subs = {'host': host, 'token': token_id}
         body = src.substitute(subs)
-        return webob.Response(body=body, status='200',
+        return webob.Response(body=body, status='200', charset='utf-8',
                               headerlist=headers)
 
     def _create_base_saml_assertion(self, context, auth):
@@ -447,8 +447,13 @@ class DomainV3(controller.V3Controller):
         :returns: list of accessible domains
 
         """
-        controller = auth_controllers.Auth()
-        return controller.get_auth_domains(request)
+        domains = self.assignment_api.list_domains_for_groups(
+            request.auth_context['group_ids'])
+        domains = domains + self.assignment_api.list_domains_for_user(
+            request.auth_context['user_id'])
+        # remove duplicates
+        domains = [dict(t) for t in set([tuple(d.items()) for d in domains])]
+        return DomainV3.wrap_collection(request.context_dict, domains)
 
 
 @dependency.requires('assignment_api', 'resource_api')
@@ -468,8 +473,14 @@ class ProjectAssignmentV3(controller.V3Controller):
         :returns: list of accessible projects
 
         """
-        controller = auth_controllers.Auth()
-        return controller.get_auth_projects(request)
+        projects = self.assignment_api.list_projects_for_groups(
+            request.auth_context['group_ids'])
+        projects = projects + self.assignment_api.list_projects_for_user(
+            request.auth_context['user_id'])
+        # remove duplicates
+        projects = [dict(t) for t in set([tuple(d.items()) for d in projects])]
+        return ProjectAssignmentV3.wrap_collection(request.context_dict,
+                                                   projects)
 
 
 @dependency.requires('federation_api')
