@@ -30,13 +30,12 @@ from six.moves import map, zip
 
 from keystone.common import driver_hints
 from keystone import exception
-from keystone.i18n import _, _LW
+from keystone.i18n import _
 
 
 LOG = log.getLogger(__name__)
 
 LDAP_VALUES = {'TRUE': True, 'FALSE': False}
-CONTROL_TREEDELETE = '1.2.840.113556.1.4.805'
 LDAP_SCOPES = {'one': ldap.SCOPE_ONELEVEL,
                'sub': ldap.SCOPE_SUBTREE}
 LDAP_DEREF = {'always': ldap.DEREF_ALWAYS,
@@ -479,14 +478,6 @@ class LDAPHandler(object):
     def modify_s(self, dn, modlist):
         raise exception.NotImplemented()  # pragma: no cover
 
-    @abc.abstractmethod
-    def delete_s(self, dn):
-        raise exception.NotImplemented()  # pragma: no cover
-
-    @abc.abstractmethod
-    def delete_ext_s(self, dn, serverctrls=None, clientctrls=None):
-        raise exception.NotImplemented()  # pragma: no cover
-
 
 class PythonLDAPHandler(LDAPHandler):
     """LDAPHandler implementation which calls the python-ldap API.
@@ -565,12 +556,6 @@ class PythonLDAPHandler(LDAPHandler):
     def modify_s(self, dn, modlist):
         return self.conn.modify_s(dn, modlist)
 
-    def delete_s(self, dn):
-        return self.conn.delete_s(dn)
-
-    def delete_ext_s(self, dn, serverctrls=None, clientctrls=None):
-        return self.conn.delete_ext_s(dn, serverctrls, clientctrls)
-
 
 def _common_ldap_initialization(url, use_tls=False, tls_cacertfile=None,
                                 tls_cacertdir=None, tls_req_cert=None,
@@ -832,14 +817,6 @@ class PooledLDAPHandler(LDAPHandler):
     def modify_s(self, conn, dn, modlist):
         return conn.modify_s(dn, modlist)
 
-    @use_conn_pool
-    def delete_s(self, conn, dn):
-        return conn.delete_s(dn)
-
-    @use_conn_pool
-    def delete_ext_s(self, conn, dn, serverctrls=None, clientctrls=None):
-        return conn.delete_ext_s(dn, serverctrls, clientctrls)
-
 
 class KeystoneLDAPHandler(LDAPHandler):
     """Convert data types and perform logging.
@@ -1037,9 +1014,9 @@ class KeystoneLDAPHandler(LDAPHandler):
                     # Exit condition no more data on server
                     break
             else:
-                LOG.warning(_LW('LDAP Server does not support paging. '
-                                'Disable paging in keystone.conf to '
-                                'avoid this message.'))
+                LOG.warning('LDAP Server does not support paging. '
+                            'Disable paging in keystone.conf to '
+                            'avoid this message.')
                 self._disable_paging()
                 break
         return res
@@ -1078,17 +1055,6 @@ class KeystoneLDAPHandler(LDAPHandler):
             for op, kind, values in ldap_modlist]
         return self.conn.modify_s(dn_utf8, ldap_modlist_utf8)
 
-    def delete_s(self, dn):
-        LOG.debug("LDAP delete: dn=%s", dn)
-        dn_utf8 = utf8_encode(dn)
-        return self.conn.delete_s(dn_utf8)
-
-    def delete_ext_s(self, dn, serverctrls=None, clientctrls=None):
-        LOG.debug('LDAP delete_ext: dn=%s serverctrls=%s clientctrls=%s',
-                  dn, serverctrls, clientctrls)
-        dn_utf8 = utf8_encode(dn)
-        return self.conn.delete_ext_s(dn_utf8, serverctrls, clientctrls)
-
     def __exit__(self, exc_type, exc_val, exc_tb):
         """Exit runtime context, unbind LDAP."""
         self.unbind_s()
@@ -1222,9 +1188,9 @@ class BaseLdap(object):
             try:
                 ldap_attr, attr_map = item.split(':')
             except ValueError:
-                LOG.warning(_LW(
+                LOG.warning(
                     'Invalid additional attribute mapping: "%s". '
-                    'Format must be <ldap_attribute>:<keystone_attribute>'),
+                    'Format must be <ldap_attribute>:<keystone_attribute>',
                     item)
                 continue
             mapping[ldap_attr] = attr_map
@@ -1282,6 +1248,8 @@ class BaseLdap(object):
                 conn.simple_bind_s()
 
             return conn
+        except ldap.INVALID_CREDENTIALS:
+            raise exception.LDAPInvalidCredentialsError()
         except ldap.SERVER_DOWN:
             raise exception.LDAPServerConnectionError(
                 url=self.LDAP_URL)
@@ -1334,11 +1302,10 @@ class BaseLdap(object):
             # logic here so it does not potentially break existing
             # deployments. We need to fix our read-write LDAP logic so
             # it does not get the ID from DN.
-            message = _LW('ID attribute %(id_attr)s for LDAP object %(dn)s '
-                          'has multiple values and therefore cannot be used '
-                          'as an ID. Will get the ID from DN instead') % (
-                              {'id_attr': self.id_attr,
-                               'dn': res[0]})
+            message = ('ID attribute %(id_attr)s for LDAP object %(dn)s '
+                       'has multiple values and therefore cannot be used '
+                       'as an ID. Will get the ID from DN instead') % (
+                           {'id_attr': self.id_attr, 'dn': res[0]})
             LOG.warning(message)
             id_val = self._dn_to_id(res[0])
         else:
@@ -1607,13 +1574,6 @@ class BaseLdap(object):
 
         return self.get(object_id)
 
-    def delete(self, object_id):
-        with self.get_connection() as conn:
-            try:
-                conn.delete_s(self._id_to_dn(object_id))
-            except ldap.NO_SUCH_OBJECT:
-                raise self._not_found(object_id)
-
     def add_member(self, member_dn, member_list_dn):
         """Add member to the member list.
 
@@ -1637,55 +1597,6 @@ class BaseLdap(object):
             except ldap.NO_SUCH_OBJECT:
                 raise self._not_found(member_list_dn)
 
-    def remove_member(self, member_dn, member_list_dn):
-        """Remove member from the member list.
-
-        :param member_dn: DN of member to be removed.
-        :param member_list_dn: DN of group from which the
-                               member will be removed.
-
-        :raises self.NotFound: If the group entry didn't exist.
-        :raises ldap.NO_SUCH_ATTRIBUTE: If the user wasn't a member.
-        """
-        with self.get_connection() as conn:
-            try:
-                mod = (ldap.MOD_DELETE, self.member_attribute, member_dn)
-                conn.modify_s(member_list_dn, [mod])
-            except ldap.NO_SUCH_OBJECT:
-                raise self._not_found(member_list_dn)
-
-    def _delete_tree_nodes(self, search_base, scope, query_params=None):
-        query = u'(objectClass=%s)' % self.object_class
-        if query_params:
-            query = (u'(&%s%s)' %
-                     (query, ''.join(['(%s=%s)'
-                                      % (k, ldap.filter.escape_filter_chars(v))
-                                      for k, v in
-                                      query_params.items()])))
-        not_deleted_nodes = []
-        with self.get_connection() as conn:
-            try:
-                nodes = conn.search_s(search_base, scope, query,
-                                      attrlist=DN_ONLY)
-            except ldap.NO_SUCH_OBJECT:
-                LOG.debug('Could not find entry with dn=%s', search_base)
-                raise self._not_found(self._dn_to_id(search_base))
-            else:
-                for node_dn, _t in nodes:
-                    try:
-                        conn.delete_s(node_dn)
-                    except ldap.NO_SUCH_OBJECT:
-                        not_deleted_nodes.append(node_dn)
-
-        if not_deleted_nodes:
-            msg = _LW('When deleting entries for %(search_base)s, '
-                      'could not delete nonexistent entries '
-                      '%(entries)s%(dots)s')
-            LOG.warning(msg,
-                        {'search_base': search_base,
-                         'entries': not_deleted_nodes[:3],
-                         'dots': '...' if len(not_deleted_nodes) > 3 else ''})
-
     def filter_query(self, hints, query=None):
         """Apply filtering to a query.
 
@@ -1923,8 +1834,3 @@ class EnabledEmuMixIn(BaseLdap):
         else:
             return super(EnabledEmuMixIn, self).update(
                 object_id, values, old_obj)
-
-    def delete(self, object_id):
-        if self.enabled_emulation:
-            self._remove_enabled(object_id)
-        super(EnabledEmuMixIn, self).delete(object_id)
