@@ -11,17 +11,7 @@
 # under the License.
 
 from keystonemiddleware import auth_token
-
-# TODO(stevemar): Remove this check once global-requirements depends on a
-# version of keystonemiddleware greater than 4.2.0. We caught this issue too
-# late in the Newton development cycle to unfreeze global requirements.
-try:
-    BaseAuthProtocol = auth_token.BaseAuthProtocol
-except AttributeError:
-    BaseAuthProtocol = auth_token._BaseAuthProtocol
-
 from oslo_log import log
-from oslo_log import versionutils
 
 from keystone.common import authorization
 from keystone.common import context
@@ -32,7 +22,7 @@ import keystone.conf
 from keystone import exception
 from keystone.federation import constants as federation_constants
 from keystone.federation import utils
-from keystone.i18n import _, _LI, _LW
+from keystone.i18n import _
 from keystone.middleware import core
 from keystone.models import token_model
 from keystone.token.providers import common
@@ -44,16 +34,18 @@ __all__ = ('AuthContextMiddleware',)
 
 
 @dependency.requires('token_provider_api')
-class AuthContextMiddleware(BaseAuthProtocol):
+class AuthContextMiddleware(auth_token.BaseAuthProtocol):
     """Build the authentication context from the request auth token."""
 
+    kwargs_to_fetch_token = True
+
     def __init__(self, app):
         bind = CONF.token.enforce_token_bind
         super(AuthContextMiddleware, self).__init__(app,
                                                     log=LOG,
                                                     enforce_token_bind=bind)
 
-    def fetch_token(self, token):
+    def fetch_token(self, token, **kwargs):
         try:
             return self.token_provider_api.validate_token(token)
         except exception.TokenNotFound:
@@ -126,16 +118,16 @@ class AuthContextMiddleware(BaseAuthProtocol):
 
         issuer = request.environ.get(CONF.tokenless_auth.issuer_attribute)
         if not issuer:
-            msg = _LI('Cannot find client issuer in env by the '
-                      'issuer attribute - %s.')
+            msg = ('Cannot find client issuer in env by the '
+                   'issuer attribute - %s.')
             LOG.info(msg, CONF.tokenless_auth.issuer_attribute)
             return False
 
         if issuer in CONF.tokenless_auth.trusted_issuer:
             return True
 
-        msg = _LI('The client issuer %(client_issuer)s does not match with '
-                  'the trusted issuer %(trusted_issuer)s')
+        msg = ('The client issuer %(client_issuer)s does not match with '
+               'the trusted issuer %(trusted_issuer)s')
         LOG.info(
             msg, {'client_issuer': issuer,
                   'trusted_issuer': CONF.tokenless_auth.trusted_issuer})
@@ -145,6 +137,21 @@ class AuthContextMiddleware(BaseAuthProtocol):
     @wsgi.middleware_exceptions
     def process_request(self, request):
         context_env = request.environ.get(core.CONTEXT_ENV, {})
+
+        # NOTE(notmorgan): This code is merged over from the admin token
+        # middleware and now emits the security warning when the
+        # conf.admin_token value is set.
+        token = request.headers.get(core.AUTH_TOKEN_HEADER)
+        if CONF.admin_token and (token == CONF.admin_token):
+            context_env['is_admin'] = True
+            LOG.warning(
+                "The use of the '[DEFAULT] admin_token' configuration"
+                "option presents a significant security risk and should "
+                "not be set. This option is deprecated in favor of using "
+                "'keystone-manage bootstrap' and will be removed in a "
+                "future release.")
+            request.environ[core.CONTEXT_ENV] = context_env
+
         if not context_env.get('is_admin', False):
             resp = super(AuthContextMiddleware, self).process_request(request)
 
@@ -167,9 +174,9 @@ class AuthContextMiddleware(BaseAuthProtocol):
         request.environ[context.REQUEST_CONTEXT_ENV] = request_context
 
         if authorization.AUTH_CONTEXT_ENV in request.environ:
-            msg = _LW('Auth context already exists in the request '
-                      'environment; it will be used for authorization '
-                      'instead of creating a new one.')
+            msg = ('Auth context already exists in the request '
+                   'environment; it will be used for authorization '
+                   'instead of creating a new one.')
             LOG.warning(msg)
             return
 
@@ -183,20 +190,6 @@ class AuthContextMiddleware(BaseAuthProtocol):
             request_context.is_admin = True
             auth_context = {}
 
-        elif CONF.admin_token and request.user_token == CONF.admin_token:
-            versionutils.report_deprecated_feature(
-                LOG,
-                _LW('build_auth_context middleware checking for the admin '
-                    'token is deprecated as of the Mitaka release and will be '
-                    'removed in the O release. If your deployment requires '
-                    'use of the admin token, update keystone-paste.ini so '
-                    'that admin_token_auth is before build_auth_context in '
-                    'the paste pipelines, otherwise remove the '
-                    'admin_token_auth middleware from the paste pipelines.'))
-
-            request_context.is_admin = True
-            auth_context = {}
-
         elif request.token_auth.has_user_token:
             request_context.auth_token = request.user_token
             ref = token_model.KeystoneToken(token_id=request.user_token,
@@ -226,6 +219,9 @@ class AuthContextMiddleware(BaseAuthProtocol):
         request_context.user_domain_id = auth_context.get('user_domain_id')
         request_context.roles = auth_context.get('roles')
 
+        is_admin_project = auth_context.get('is_admin_project', True)
+        request_context.is_admin_project = is_admin_project
+
         project_domain_id = auth_context.get('project_domain_id')
         request_context.project_domain_id = project_domain_id
 
