@@ -16,12 +16,10 @@ from oslo_utils import reflection
 from oslo_utils import timeutils
 import six
 
-import keystone.conf
 from keystone import exception
 from keystone.federation import constants
 from keystone.i18n import _
 
-CONF = keystone.conf.CONF
 # supported token versions
 V2 = 'v2.0'
 V3 = 'v3.0'
@@ -194,13 +192,14 @@ class KeystoneToken(dict):
 
     @property
     def is_admin_project(self):
+        # Prevent domain scoped tokens from acting as is_admin_project
         if self.domain_scoped:
-            # Currently, domain scoped tokens cannot act as is_admin_project
             return False
-
-        # True gets returned by default for compatibility with older versions
-        # TODO(henry-nash): This seems inherently dangerous, and we should
-        # investigate how we can default this to False.
+        # TODO(ayoung/edmondsw): Having is_admin_project default to True is
+        # essential for fixing bug #968696. If an admin project is not
+        # configured, we can add checks for is_admin_project:True and not
+        # block anyone that hasn't configured an admin_project. Do not change
+        # this until we can assume admin_project is actually set
         return self.get('is_admin_project', True)
 
     @property
