@@ -20,9 +20,9 @@ from oslo_utils import timeutils
 from six.moves import http_client
 from six.moves.urllib import parse as urlparse
 
+from keystone.common import authorization
 from keystone.common import controller
 from keystone.common import dependency
-from keystone.common import utils
 from keystone.common import validation
 from keystone.common import wsgi
 import keystone.conf
@@ -93,7 +93,7 @@ class ConsumerCrudV3(controller.V3Controller):
 
     @controller.protected()
     def delete_consumer(self, request, consumer_id):
-        user_token_ref = utils.get_token_ref(request.context_dict)
+        user_token_ref = authorization.get_token_ref(request.context_dict)
         payload = {'user_id': user_token_ref.user_id,
                    'consumer_id': consumer_id}
         _emit_user_oauth_consumer_token_invalidate(payload)
@@ -270,7 +270,7 @@ class OAuthControllerV3(controller.V3Controller):
                   % {'key': token_ref['id'],
                      'secret': token_ref['request_secret']})
 
-        if CONF.oauth1.request_token_duration:
+        if CONF.oauth1.request_token_duration > 0:
             expiry_bit = '&oauth_expires_at=%s' % token_ref['expires_at']
             result += expiry_bit
 
@@ -361,7 +361,7 @@ class OAuthControllerV3(controller.V3Controller):
                   % {'key': token_ref['id'],
                      'secret': token_ref['access_secret']})
 
-        if CONF.oauth1.access_token_duration:
+        if CONF.oauth1.access_token_duration > 0:
             expiry_bit = '&oauth_expires_at=%s' % (token_ref['expires_at'])
             result += expiry_bit
 
@@ -404,7 +404,7 @@ class OAuthControllerV3(controller.V3Controller):
             authed_roles.add(role['id'])
 
         # verify the authorizing user has the roles
-        user_token = utils.get_token_ref(request.context_dict)
+        user_token = authorization.get_token_ref(request.context_dict)
         user_id = user_token.user_id
         project_id = req_token['requested_project_id']
         user_roles = self.assignment_api.get_roles_for_user_and_project(
