@@ -15,7 +15,6 @@ from oslo_log import log
 from keystone.common import driver_hints
 from keystone.common import sql
 from keystone import exception
-from keystone.i18n import _LE, _LW
 from keystone.resource.backends import base
 
 
@@ -23,6 +22,8 @@ LOG = log.getLogger(__name__)
 
 
 class Resource(base.ResourceDriverBase):
+    # TODO(morgan): Merge all of this code into the manager, Resource backend
+    # is only SQL. There is no configurable driver.
 
     def default_assignment_driver(self):
         return 'sql'
@@ -133,9 +134,9 @@ class Resource(base.ResourceDriverBase):
                 children_ids = set()
                 for ref in children:
                     if ref['id'] in examined:
-                        msg = _LE('Circular reference or a repeated '
-                                  'entry found in projects hierarchy - '
-                                  '%(project_id)s.')
+                        msg = ('Circular reference or a repeated '
+                               'entry found in projects hierarchy - '
+                               '%(project_id)s.')
                         LOG.error(msg, {'project_id': ref['id']})
                         return
                     children_ids.add(ref['id'])
@@ -152,9 +153,9 @@ class Resource(base.ResourceDriverBase):
             examined = set()
             while project.get('parent_id') is not None:
                 if project['id'] in examined:
-                    msg = _LE('Circular reference or a repeated '
-                              'entry found in projects hierarchy - '
-                              '%(project_id)s.')
+                    msg = ('Circular reference or a repeated '
+                           'entry found in projects hierarchy - '
+                           '%(project_id)s.')
                     LOG.error(msg, {'project_id': project['id']})
                     return
 
@@ -214,12 +215,12 @@ class Resource(base.ResourceDriverBase):
             for project_id in project_ids:
                 if (project_id not in project_ids_from_bd or
                         project_id == base.NULL_DOMAIN_ID):
-                    LOG.warning(_LW('Project %s does not exist and was not '
-                                    'deleted.') % project_id)
+                    LOG.warning('Project %s does not exist and was not '
+                                'deleted.', project_id)
             query.delete(synchronize_session=False)
 
 
-class Project(sql.ModelBase, sql.DictBase):
+class Project(sql.ModelBase, sql.ModelDictMixinWithExtras):
     # NOTE(henry-nash): From the manager and above perspective, the domain_id
     # is nullable.  However, to ensure uniqueness in multi-process
     # configurations, it is better to still use the sql uniqueness constraint.
