@@ -24,9 +24,10 @@ from keystone.common import manager
 from keystone.common import utils
 import keystone.conf
 from keystone import exception
-from keystone.i18n import _, _LE, _LW
+from keystone.i18n import _
 from keystone import notifications
 from keystone.resource.backends import base
+from keystone.resource.backends import sql as resource_sql
 from keystone.token import provider as token_provider
 
 CONF = keystone.conf.CONF
@@ -36,7 +37,7 @@ MEMOIZE = cache.get_memoization_decorator(group='resource')
 
 @dependency.provider('resource_api')
 @dependency.requires('assignment_api', 'credential_api', 'domain_config_api',
-                     'identity_api', 'revoke_api', 'trust_api')
+                     'identity_api', 'trust_api')
 class Manager(manager.Manager):
     """Default pivot point for the Resource backend.
 
@@ -51,8 +52,12 @@ class Manager(manager.Manager):
     _PROJECT = 'project'
 
     def __init__(self):
-        resource_driver = CONF.resource.driver
-        super(Manager, self).__init__(resource_driver)
+        # NOTE(morgan): The resource driver must be SQL. This is because there
+        # is a FK between identity and resource. Almost every deployment uses
+        # SQL Identity in some form. Even if SQL Identity is not used, there
+        # is almost no reason to have non-SQL Resource. Keystone requires
+        # SQL in a number of ways, this simply codifies it plainly for resource
+        self.driver = resource_sql.Resource()
 
     def _get_hierarchy_depth(self, parents_list):
         return len(parents_list) + 1
@@ -332,6 +337,11 @@ class Manager(manager.Manager):
 
             notifications.Audit.disabled(self._PROJECT, project_id,
                                          public=False)
+            # Drop the computed assignments if the project is being disabled.
+            # This ensures an accurate list of projects is returned when
+            # listing projects/domains for a user based on role assignments.
+            assignment.COMPUTED_ASSIGNMENTS_REGION.invalidate()
+
         if cascade:
             self._only_allow_enabled_to_update_cascade(project,
                                                        original_project)
@@ -686,9 +696,9 @@ class Manager(manager.Manager):
         result can be returned in response to a domain API call.
         """
         if not project_ref['is_domain']:
-            LOG.error(_LE('Asked to convert a non-domain project into a '
-                          'domain - Domain: %(domain_id)s, Project ID: '
-                          '%(id)s, Project Name: %(project_name)s'),
+            LOG.error('Asked to convert a non-domain project into a '
+                      'domain - Domain: %(domain_id)s, Project ID: '
+                      '%(id)s, Project Name: %(project_name)s',
                       {'domain_id': project_ref['domain_id'],
                        'id': project_ref['id'],
                        'project_name': project_ref['name']})
@@ -757,9 +767,6 @@ class Manager(manager.Manager):
                   'first.'))
 
         self._delete_domain_contents(domain_id)
-        notifications.Audit.internal(
-            notifications.DOMAIN_DELETED, domain_id
-        )
         self._delete_project(domain_id, initiator)
         try:
             self.get_domain.invalidate(self, domain_id)
@@ -783,8 +790,8 @@ class Manager(manager.Manager):
         """
         def _delete_projects(project, projects, examined):
             if project['id'] in examined:
-                msg = _LE('Circular reference or a repeated entry found '
-                          'projects hierarchy - %(project_id)s.')
+                msg = ('Circular reference or a repeated entry found '
+                       'projects hierarchy - %(project_id)s.')
                 LOG.error(msg, {'project_id': project['id']})
                 return
 
@@ -848,15 +855,15 @@ class Manager(manager.Manager):
             }
             self.create_domain(CONF.identity.default_domain_id,
                                default_domain_attrs)
-            LOG.warning(_LW(
+            LOG.warning(
                 'The default domain was created automatically to contain V2 '
                 'resources. This is deprecated in the M release and will not '
                 'be supported in the O release. Create the default domain '
-                'manually or use the keystone-manage bootstrap command.'))
+                'manually or use the keystone-manage bootstrap command.')
         except exception.Conflict:
             LOG.debug('The default domain already exists.')
         except Exception:
-            LOG.error(_LE('Failed to create the default domain.'))
+            LOG.error('Failed to create the default domain.')
             raise
 
     def _require_matching_domain_id(self, new_ref, orig_ref):
@@ -894,6 +901,11 @@ class DomainConfigManager(manager.Manager):
 
     driver_namespace = 'keystone.resource.domain_config'
 
+    # We explicitly state each whitelisted option instead of pulling all ldap
+    # options from CONF and selectively pruning them to prevent a security
+    # lapse. That way if a new ldap CONF key/value were to be added it wouldn't
+    # automatically be added to the whitelisted options unless that is what was
+    # intended. In which case, we explicitly add it to the list ourselves.
     whitelisted_options = {
         'identity': ['driver', 'list_limit'],
         'ldap': [
@@ -909,11 +921,11 @@ class DomainConfigManager(manager.Manager):
             'user_enabled_emulation_use_group_config',
             'user_additional_attribute_mapping', 'group_tree_dn',
             'group_filter', 'group_objectclass', 'group_id_attribute',
-            'group_name_attribute', 'group_member_attribute',
-            'group_desc_attribute', 'group_attribute_ignore',
-            'group_additional_attribute_mapping', 'tls_cacertfile',
-            'tls_cacertdir', 'use_tls', 'tls_req_cert', 'use_pool',
-            'pool_size', 'pool_retry_max', 'pool_retry_delay',
+            'group_name_attribute', 'group_members_are_ids',
+            'group_member_attribute', 'group_desc_attribute',
+            'group_attribute_ignore', 'group_additional_attribute_mapping',
+            'tls_cacertfile', 'tls_cacertdir', 'use_tls', 'tls_req_cert',
+            'use_pool', 'pool_size', 'pool_retry_max', 'pool_retry_delay',
             'pool_connection_timeout', 'pool_connection_lifetime',
             'use_auth_pool', 'auth_pool_size', 'auth_pool_connection_lifetime'
         ]
@@ -1031,9 +1043,9 @@ class DomainConfigManager(manager.Manager):
             # there is only one option in the answer (and that it's the right
             # one) - if not, something has gone wrong and we raise an error
             if len(the_list) > 1 or the_list[0]['option'] != req_option:
-                LOG.error(_LE('Unexpected results in response for domain '
-                              'config - %(count)s responses, first option is '
-                              '%(option)s, expected option %(expected)s'),
+                LOG.error('Unexpected results in response for domain '
+                          'config - %(count)s responses, first option is '
+                          '%(option)s, expected option %(expected)s',
                           {'count': len(the_list), 'option': list[0]['option'],
                            'expected': req_option})
                 raise exception.UnexpectedError(
@@ -1337,21 +1349,21 @@ class DomainConfigManager(manager.Manager):
                 each_whitelisted['value'] = (
                     each_whitelisted['value'] % sensitive_dict)
             except KeyError:
-                warning_msg = _LW(
+                warning_msg = (
                     'Found what looks like an unmatched config option '
                     'substitution reference - domain: %(domain)s, group: '
                     '%(group)s, option: %(option)s, value: %(value)s. Perhaps '
                     'the config option to which it refers has yet to be '
                     'added?')
             except (ValueError, TypeError):
-                warning_msg = _LW(
+                warning_msg = (
                     'Found what looks like an incorrectly constructed '
                     'config option substitution reference - domain: '
                     '%(domain)s, group: %(group)s, option: %(option)s, '
                     'value: %(value)s.')
 
             if warning_msg:
-                LOG.warning(warning_msg % {
+                LOG.warning(warning_msg, {
                     'domain': domain_id,
                     'group': each_whitelisted['group'],
                     'option': each_whitelisted['option'],
