@@ -14,7 +14,6 @@ import uuid
 
 from six.moves import range
 
-import keystone.conf
 from keystone.credential.providers import fernet as credential_provider
 from keystone.tests import unit
 from keystone.tests.unit import default_fixtures
@@ -22,9 +21,6 @@ from keystone.tests.unit import ksfixtures
 from keystone.tests.unit.ksfixtures import database
 
 
-CONF = keystone.conf.CONF
-
-
 class SqlTests(unit.SQLDriverOverrides, unit.TestCase):
 
     def setUp(self):
