@@ -14,11 +14,13 @@
 
 import uuid
 
-from oslo_config import cfg
 from six.moves import range
 
+import keystone.conf
+from keystone import exception
 
-CONF = cfg.CONF
+
+CONF = keystone.conf.CONF
 
 
 class FilterTests(object):
@@ -93,13 +95,13 @@ class FilterTests(object):
             # The manager layer creates the ID for users and groups
             new_entity = self._create_entity(entity_type)(new_entity)
         else:
-            new_entity['id'] = '0000' + uuid.uuid4().hex
+            new_entity['id'] = uuid.uuid4().hex
             self._create_entity(entity_type)(new_entity['id'], new_entity)
         return new_entity
 
     def _create_test_data(self, entity_type, number, domain_id=None,
                           name_dict=None):
-        """Create entity test data
+        """Create entity test data.
 
         :param entity_type: type of entity to create, e.g. 'user', group' etc.
         :param number: number of entities to create,
@@ -121,4 +123,9 @@ class FilterTests(object):
 
     def _delete_test_data(self, entity_type, entity_list):
         for entity in entity_list:
-            self._delete_entity(entity_type)(entity['id'])
+            try:
+                self._delete_entity(entity_type)(entity['id'])
+            except exception.Forbidden:
+                # Note(knikolla): Some identity backends such as LDAP are
+                # read only
+                break
