@@ -18,7 +18,7 @@ import uuid
 import fixtures
 import ldap.dn
 import mock
-from testtools import matchers
+from oslo_config import fixture as config_fixture
 
 from keystone.common import driver_hints
 import keystone.conf
@@ -208,7 +208,7 @@ class LDAPDeleteTreeTest(unit.TestCase):
 
         self.useFixture(
             ldapdb.LDAPDatabase(dbclass=fakeldap.FakeLdapNoSubtreeDelete))
-        self.useFixture(database.Database(self.sql_driver_version_overrides))
+        self.useFixture(database.Database())
 
         self.load_backends()
         self.load_fixtures(default_fixtures)
@@ -222,74 +222,20 @@ class LDAPDeleteTreeTest(unit.TestCase):
         config_files.append(unit.dirs.tests_conf('backend_ldap.conf'))
         return config_files
 
-    def test_delete_tree(self):
-        """Test manually deleting a tree.
-
-        Few LDAP servers support CONTROL_DELETETREE.  This test
-        exercises the alternate code paths in BaseLdap.delete_tree.
-
-        """
-        conn = self.identity_api.user.get_connection()
-        id_attr = self.identity_api.user.id_attr
-        objclass = self.identity_api.user.object_class.lower()
-        tree_dn = self.identity_api.user.tree_dn
-
-        def create_entry(name, parent_dn=None):
-            if not parent_dn:
-                parent_dn = tree_dn
-            dn = '%s=%s,%s' % (id_attr, name, parent_dn)
-            attrs = [('objectclass', [objclass, 'ldapsubentry']),
-                     (id_attr, [name])]
-            conn.add_s(dn, attrs)
-            return dn
-
-        # create 3 entries like this:
-        # cn=base
-        # cn=child,cn=base
-        # cn=grandchild,cn=child,cn=base
-        # then attempt to delete_tree(cn=base)
-        base_id = 'base'
-        base_dn = create_entry(base_id)
-        child_dn = create_entry('child', base_dn)
-        grandchild_dn = create_entry('grandchild', child_dn)
-
-        # verify that the three entries were created
-        scope = ldap.SCOPE_SUBTREE
-        filt = '(|(objectclass=*)(objectclass=ldapsubentry))'
-        entries = conn.search_s(base_dn, scope, filt,
-                                attrlist=common_ldap.DN_ONLY)
-        self.assertThat(entries, matchers.HasLength(3))
-        sort_ents = sorted([e[0] for e in entries], key=len, reverse=True)
-        self.assertEqual([grandchild_dn, child_dn, base_dn], sort_ents)
-
-        # verify that a non-leaf node can't be deleted directly by the
-        # LDAP server
-        self.assertRaises(ldap.NOT_ALLOWED_ON_NONLEAF,
-                          conn.delete_s, base_dn)
-        self.assertRaises(ldap.NOT_ALLOWED_ON_NONLEAF,
-                          conn.delete_s, child_dn)
-
-        # call our delete_tree implementation
-        self.identity_api.user.delete_tree(base_id)
-        self.assertRaises(ldap.NO_SUCH_OBJECT,
-                          conn.search_s, base_dn, ldap.SCOPE_BASE)
-        self.assertRaises(ldap.NO_SUCH_OBJECT,
-                          conn.search_s, child_dn, ldap.SCOPE_BASE)
-        self.assertRaises(ldap.NO_SUCH_OBJECT,
-                          conn.search_s, grandchild_dn, ldap.SCOPE_BASE)
-
 
 class MultiURLTests(unit.TestCase):
     """Test for setting multiple LDAP URLs."""
 
-    def test_multiple_urls_with_comma_no_conn_pool(self):
+    @mock.patch.object(common_ldap.KeystoneLDAPHandler, 'simple_bind_s')
+    def test_multiple_urls_with_comma_no_conn_pool(self, mock_ldap_bind):
         urls = 'ldap://localhost,ldap://backup.localhost'
         self.config_fixture.config(group='ldap', url=urls, use_pool=False)
         base_ldap = common_ldap.BaseLdap(CONF)
         ldap_connection = base_ldap.get_connection()
         self.assertEqual(urls, ldap_connection.conn.conn._uri)
 
-    def test_multiple_urls_with_comma_with_conn_pool(self):
+    @mock.patch.object(common_ldap.KeystoneLDAPHandler, 'simple_bind_s')
+    def test_multiple_urls_with_comma_with_conn_pool(self, mock_ldap_bind):
         urls = 'ldap://localhost,ldap://backup.localhost'
         self.config_fixture.config(group='ldap', url=urls, use_pool=True)
         base_ldap = common_ldap.BaseLdap(CONF)
@@ -297,9 +243,68 @@ class MultiURLTests(unit.TestCase):
         self.assertEqual(urls, ldap_connection.conn.conn_pool.uri)
 
 
-class SslTlsTest(unit.TestCase):
+class LDAPConnectionTimeoutTest(unit.TestCase):
+    """Test for Network Connection timeout on LDAP URL connection."""
+
+    @mock.patch.object(common_ldap.KeystoneLDAPHandler, 'simple_bind_s')
+    def test_connectivity_timeout_no_conn_pool(self, mock_ldap_bind):
+        url = 'ldap://localhost'
+        conn_timeout = 1  # 1 second
+        self.config_fixture.config(group='ldap',
+                                   url=url,
+                                   connection_timeout=conn_timeout,
+                                   use_pool=False)
+        base_ldap = common_ldap.BaseLdap(CONF)
+        ldap_connection = base_ldap.get_connection()
+        self.assertIsInstance(ldap_connection.conn,
+                              common_ldap.PythonLDAPHandler)
+
+        # Ensure that the Network Timeout option is set.
+        # Also ensure that the URL is set.
+        #
+        # We will not verify if an LDAP bind returns the timeout
+        # exception as that would fall under the realm of
+        # integration testing. If the LDAP option is set properly,
+        # and we get back a valid connection URI then that should
+        # suffice for this unit test.
+        self.assertEqual(conn_timeout,
+                         ldap.get_option(ldap.OPT_NETWORK_TIMEOUT))
+        self.assertEqual(url, ldap_connection.conn.conn._uri)
+
+    @mock.patch.object(common_ldap.KeystoneLDAPHandler, 'simple_bind_s')
+    def test_connectivity_timeout_with_conn_pool(self, mock_ldap_bind):
+        url = 'ldap://localhost'
+        conn_timeout = 1  # 1 second
+        self.config_fixture.config(group='ldap',
+                                   url=url,
+                                   pool_connection_timeout=conn_timeout,
+                                   use_pool=True,
+                                   pool_retry_max=1)
+        base_ldap = common_ldap.BaseLdap(CONF)
+        ldap_connection = base_ldap.get_connection()
+        self.assertIsInstance(ldap_connection.conn,
+                              common_ldap.PooledLDAPHandler)
+
+        # Ensure that the Network Timeout option is set.
+        # Also ensure that the URL is set.
+        #
+        # We will not verify if an LDAP bind returns the timeout
+        # exception as that would fall under the realm of
+        # integration testing. If the LDAP option is set properly,
+        # and we get back a valid connection URI then that should
+        # suffice for this unit test.
+        self.assertEqual(conn_timeout,
+                         ldap.get_option(ldap.OPT_NETWORK_TIMEOUT))
+        self.assertEqual(url, ldap_connection.conn.conn_pool.uri)
+
+
+class SslTlsTest(unit.BaseTestCase):
     """Test for the SSL/TLS functionality in keystone.common.ldap.core."""
 
+    def setUp(self):
+        super(SslTlsTest, self).setUp()
+        self.config_fixture = self.useFixture(config_fixture.Config(CONF))
+
     @mock.patch.object(common_ldap.KeystoneLDAPHandler, 'simple_bind_s')
     @mock.patch.object(ldap.ldapobject.LDAPObject, 'start_tls_s')
     def _init_ldap_connection(self, config, mock_ldap_one, mock_ldap_two):
@@ -371,7 +376,7 @@ class LDAPPagedResultsTest(unit.TestCase):
         super(LDAPPagedResultsTest, self).setUp()
 
         self.useFixture(ldapdb.LDAPDatabase())
-        self.useFixture(database.Database(self.sql_driver_version_overrides))
+        self.useFixture(database.Database())
 
         self.load_backends()
         self.load_fixtures(default_fixtures)
@@ -506,11 +511,12 @@ class CommonLdapTestCase(unit.BaseTestCase):
             self.assertEqual(user_name, py_result[0][1]['user_name'][0])
 
 
-class LDAPFilterQueryCompositionTest(unit.TestCase):
+class LDAPFilterQueryCompositionTest(unit.BaseTestCase):
     """These test cases test LDAP filter generation."""
 
     def setUp(self):
         super(LDAPFilterQueryCompositionTest, self).setUp()
+        self.config_fixture = self.useFixture(config_fixture.Config(CONF))
 
         self.base_ldap = common_ldap.BaseLdap(self.config_fixture.conf)
 
