@@ -17,63 +17,69 @@ import uuid
 import mock
 
 from keystone import auth
+from keystone.auth.plugins import base
 from keystone import exception
 from keystone.tests import unit
+from keystone.tests.unit.ksfixtures import auth_plugins
 
 
 # for testing purposes only
 METHOD_NAME = 'simple_challenge_response'
+METHOD_OPTS = {
+    METHOD_NAME:
+        'keystone.tests.unit.test_auth_plugin.SimpleChallengeResponse',
+}
 EXPECTED_RESPONSE = uuid.uuid4().hex
 DEMO_USER_ID = uuid.uuid4().hex
 
 
-class SimpleChallengeResponse(auth.AuthMethodHandler):
-    def authenticate(self, context, auth_payload, user_context):
+class SimpleChallengeResponse(base.AuthMethodHandler):
+    def authenticate(self, context, auth_payload):
+        response_data = {}
         if 'response' in auth_payload:
             if auth_payload['response'] != EXPECTED_RESPONSE:
                 raise exception.Unauthorized('Wrong answer')
-            user_context['user_id'] = DEMO_USER_ID
+
+            response_data['user_id'] = DEMO_USER_ID
+            return base.AuthHandlerResponse(status=True, response_body=None,
+                                            response_data=response_data)
         else:
-            return {"challenge": "What's the name of your high school?"}
+            return base.AuthHandlerResponse(
+                status=False,
+                response_body={
+                    "challenge": "What's the name of your high school?"},
+                response_data=None)
 
 
 class TestAuthPlugin(unit.SQLDriverOverrides, unit.TestCase):
     def setUp(self):
         super(TestAuthPlugin, self).setUp()
-        self.load_backends()
-
         self.api = auth.controllers.Auth()
 
-    def config_overrides(self):
-        super(TestAuthPlugin, self).config_overrides()
-        method_opts = {
-            METHOD_NAME:
-                'keystone.tests.unit.test_auth_plugin.SimpleChallengeResponse',
-        }
-
-        self.auth_plugin_config_override(
-            methods=['external', 'password', 'token', METHOD_NAME],
-            **method_opts)
-
     def test_unsupported_auth_method(self):
         method_name = uuid.uuid4().hex
         auth_data = {'methods': [method_name]}
         auth_data[method_name] = {'test': 'test'}
         auth_data = {'identity': auth_data}
         self.assertRaises(exception.AuthMethodNotSupported,
-                          auth.controllers.AuthInfo.create,
-                          None,
+                          auth.core.AuthInfo.create,
                           auth_data)
 
     def test_addition_auth_steps(self):
+        self.useFixture(
+            auth_plugins.ConfigAuthPlugins(self.config_fixture,
+                                           methods=[METHOD_NAME],
+                                           **METHOD_OPTS))
+        self.useFixture(auth_plugins.LoadAuthPlugins(METHOD_NAME))
+
         auth_data = {'methods': [METHOD_NAME]}
         auth_data[METHOD_NAME] = {
             'test': 'test'}
         auth_data = {'identity': auth_data}
-        auth_info = auth.controllers.AuthInfo.create(None, auth_data)
-        auth_context = {'extras': {}, 'method_names': []}
+        auth_info = auth.core.AuthInfo.create(auth_data)
+        auth_context = auth.core.AuthContext(extras={}, method_names=[])
         try:
-            self.api.authenticate({'environment': {}}, auth_info, auth_context)
+            self.api.authenticate(self.make_request(), auth_info, auth_context)
         except exception.AdditionalAuthRequired as e:
             self.assertIn('methods', e.authentication)
             self.assertIn(METHOD_NAME, e.authentication['methods'])
@@ -85,9 +91,9 @@ class TestAuthPlugin(unit.SQLDriverOverrides, unit.TestCase):
         auth_data[METHOD_NAME] = {
             'response': EXPECTED_RESPONSE}
         auth_data = {'identity': auth_data}
-        auth_info = auth.controllers.AuthInfo.create(None, auth_data)
-        auth_context = {'extras': {}, 'method_names': []}
-        self.api.authenticate({'environment': {}}, auth_info, auth_context)
+        auth_info = auth.core.AuthInfo.create(auth_data)
+        auth_context = auth.core.AuthContext(extras={}, method_names=[])
+        self.api.authenticate(self.make_request(), auth_info, auth_context)
         self.assertEqual(DEMO_USER_ID, auth_context['user_id'])
 
         # test incorrect response
@@ -95,21 +101,21 @@ class TestAuthPlugin(unit.SQLDriverOverrides, unit.TestCase):
         auth_data[METHOD_NAME] = {
             'response': uuid.uuid4().hex}
         auth_data = {'identity': auth_data}
-        auth_info = auth.controllers.AuthInfo.create(None, auth_data)
-        auth_context = {'extras': {}, 'method_names': []}
+        auth_info = auth.core.AuthInfo.create(auth_data)
+        auth_context = auth.core.AuthContext(extras={}, method_names=[])
         self.assertRaises(exception.Unauthorized,
                           self.api.authenticate,
-                          {'environment': {}},
+                          self.make_request(),
                           auth_info,
                           auth_context)
 
     def test_duplicate_method(self):
         # Having the same method twice doesn't cause load_auth_methods to fail.
-        self.auth_plugin_config_override(
-            methods=['external', 'external'])
-        self.clear_auth_plugin_registry()
-        auth.controllers.load_auth_methods()
-        self.assertIn('external', auth.controllers.AUTH_METHODS)
+        self.useFixture(
+            auth_plugins.ConfigAuthPlugins(self.config_fixture,
+                                           ['external', 'external']))
+        auth.core.load_auth_methods()
+        self.assertIn('external', auth.core.AUTH_METHODS)
 
 
 class TestAuthPluginDynamicOptions(TestAuthPlugin):
@@ -128,8 +134,6 @@ class TestAuthPluginDynamicOptions(TestAuthPlugin):
 class TestMapped(unit.TestCase):
     def setUp(self):
         super(TestMapped, self).setUp()
-        self.load_backends()
-
         self.api = auth.controllers.Auth()
 
     def config_files(self):
@@ -137,54 +141,56 @@ class TestMapped(unit.TestCase):
         config_files.append(unit.dirs.tests_conf('test_auth_plugin.conf'))
         return config_files
 
-    def auth_plugin_config_override(self, methods=None, **method_classes):
-        # Do not apply the auth plugin overrides so that the config file is
-        # tested
-        pass
-
     def _test_mapped_invocation_with_method_name(self, method_name):
         with mock.patch.object(auth.plugins.mapped.Mapped,
                                'authenticate',
                                return_value=None) as authenticate:
-            context = {'environment': {}}
+            request = self.make_request()
             auth_data = {
                 'identity': {
                     'methods': [method_name],
                     method_name: {'protocol': method_name},
                 }
             }
-            auth_info = auth.controllers.AuthInfo.create(context, auth_data)
-            auth_context = {'extras': {},
-                            'method_names': [],
-                            'user_id': uuid.uuid4().hex}
-            self.api.authenticate(context, auth_info, auth_context)
+            auth_info = auth.core.AuthInfo.create(auth_data)
+            auth_context = auth.core.AuthContext(
+                extras={},
+                method_names=[],
+                user_id=uuid.uuid4().hex)
+            self.api.authenticate(request, auth_info, auth_context)
             # make sure Mapped plugin got invoked with the correct payload
-            ((context, auth_payload, auth_context),
+            ((context, auth_payload),
              kwargs) = authenticate.call_args
             self.assertEqual(method_name, auth_payload['protocol'])
 
     def test_mapped_with_remote_user(self):
+        method_name = 'saml2'
+        auth_data = {'methods': [method_name]}
+        # put the method name in the payload so its easier to correlate
+        # method name with payload
+        auth_data[method_name] = {'protocol': method_name}
+        auth_data = {'identity': auth_data}
+
+        auth_context = auth.core.AuthContext(
+            extras={},
+            method_names=[],
+            user_id=uuid.uuid4().hex)
+
+        self.useFixture(auth_plugins.LoadAuthPlugins(method_name))
+
         with mock.patch.object(auth.plugins.mapped.Mapped,
                                'authenticate',
                                return_value=None) as authenticate:
-            # external plugin should fail and pass to mapped plugin
-            method_name = 'saml2'
-            auth_data = {'methods': [method_name]}
-            # put the method name in the payload so its easier to correlate
-            # method name with payload
-            auth_data[method_name] = {'protocol': method_name}
-            auth_data = {'identity': auth_data}
-            auth_info = auth.controllers.AuthInfo.create(None, auth_data)
-            auth_context = {'extras': {},
-                            'method_names': [],
-                            'user_id': uuid.uuid4().hex}
-            environment = {'environment': {'REMOTE_USER': 'foo@idp.com'}}
-            self.api.authenticate(environment, auth_info, auth_context)
+            auth_info = auth.core.AuthInfo.create(auth_data)
+            request = self.make_request(environ={'REMOTE_USER': 'foo@idp.com'})
+            self.api.authenticate(request, auth_info, auth_context)
             # make sure Mapped plugin got invoked with the correct payload
-            ((context, auth_payload, auth_context),
+            ((context, auth_payload),
              kwargs) = authenticate.call_args
             self.assertEqual(method_name, auth_payload['protocol'])
 
     def test_supporting_multiple_methods(self):
-        for method_name in ['saml2', 'openid', 'x509']:
+        method_names = ('saml2', 'openid', 'x509', 'mapped')
+        self.useFixture(auth_plugins.LoadAuthPlugins(*method_names))
+        for method_name in method_names:
             self._test_mapped_invocation_with_method_name(method_name)
