@@ -19,8 +19,8 @@ import uuid
 import fixtures
 import mock
 import oslo_config.fixture
+from oslo_db.sqlalchemy import migration
 from oslo_log import log
-from oslotest import mockpatch
 from six.moves import configparser
 from six.moves import range
 from testtools import matchers
@@ -62,25 +62,13 @@ class CliTestCase(unit.SQLDriverOverrides, unit.TestCase):
         self.load_backends()
         cli.TokenFlush.main()
 
-    # NOTE(ravelar): the following method tests that the token_flush command,
-    # when used in conjunction with an unsupported token driver like kvs,
-    # will yield a LOG.warning message informing the user that the
-    # command had no effect.
-    def test_token_flush_excepts_not_implemented_and_logs_warning(self):
-        self.useFixture(database.Database())
-        self.load_backends()
-        self.config_fixture.config(group='token', driver='kvs')
-        log_info = self.useFixture(fixtures.FakeLogger(level=log.WARN))
-        cli.TokenFlush.main()
-        self.assertIn("token_flush command had no effect", log_info.output)
-
 
 class CliNoConfigTestCase(unit.BaseTestCase):
 
     def setUp(self):
         self.config_fixture = self.useFixture(oslo_config.fixture.Config(CONF))
         self.config_fixture.register_cli_opt(cli.command_opt)
-        self.useFixture(mockpatch.Patch(
+        self.useFixture(fixtures.MockPatch(
             'oslo_config.cfg.find_config_files', return_value=[]))
         super(CliNoConfigTestCase, self).setUp()
 
@@ -89,7 +77,7 @@ class CliNoConfigTestCase(unit.BaseTestCase):
         class FakeConfCommand(object):
             def __init__(self):
                 self.cmd_class = mock.Mock()
-        self.useFixture(mockpatch.PatchObject(
+        self.useFixture(fixtures.MockPatchObject(
             CONF, 'command', FakeConfCommand()))
 
         self.logging = self.useFixture(fixtures.FakeLogger(level=log.WARN))
@@ -635,6 +623,7 @@ class CliDBSyncTestCase(unit.BaseTestCase):
     class FakeConfCommand(object):
         def __init__(self, parent):
             self.extension = False
+            self.check = parent.command_check
             self.expand = parent.command_expand
             self.migrate = parent.command_migrate
             self.contract = parent.command_contract
@@ -648,6 +637,7 @@ class CliDBSyncTestCase(unit.BaseTestCase):
         upgrades.expand_schema = mock.Mock()
         upgrades.migrate_data = mock.Mock()
         upgrades.contract_schema = mock.Mock()
+        self.command_check = False
         self.command_expand = False
         self.command_migrate = False
         self.command_contract = False
@@ -663,7 +653,7 @@ class CliDBSyncTestCase(unit.BaseTestCase):
                 self.assertFalse(func.called)
 
     def test_db_sync(self):
-        self.useFixture(mockpatch.PatchObject(
+        self.useFixture(fixtures.MockPatchObject(
             CONF, 'command', self.FakeConfCommand(self)))
         cli.DbSync.main()
         self._assert_correct_call(
@@ -671,25 +661,36 @@ class CliDBSyncTestCase(unit.BaseTestCase):
 
     def test_db_sync_expand(self):
         self.command_expand = True
-        self.useFixture(mockpatch.PatchObject(
+        self.useFixture(fixtures.MockPatchObject(
             CONF, 'command', self.FakeConfCommand(self)))
         cli.DbSync.main()
         self._assert_correct_call(upgrades.expand_schema)
 
     def test_db_sync_migrate(self):
         self.command_migrate = True
-        self.useFixture(mockpatch.PatchObject(
+        self.useFixture(fixtures.MockPatchObject(
             CONF, 'command', self.FakeConfCommand(self)))
         cli.DbSync.main()
         self._assert_correct_call(upgrades.migrate_data)
 
     def test_db_sync_contract(self):
         self.command_contract = True
-        self.useFixture(mockpatch.PatchObject(
+        self.useFixture(fixtures.MockPatchObject(
             CONF, 'command', self.FakeConfCommand(self)))
         cli.DbSync.main()
         self._assert_correct_call(upgrades.contract_schema)
 
+    @mock.patch('keystone.cmd.cli.upgrades.get_db_version')
+    def test_db_sync_check_when_database_is_empty(self, mocked_get_db_version):
+        e = migration.exception.DbMigrationError("Invalid version")
+        mocked_get_db_version.side_effect = e
+        checker = cli.DbSync()
+
+        log_info = self.useFixture(fixtures.FakeLogger(level=log.INFO))
+        status = checker.check_db_sync_status()
+        self.assertIn("not currently under version control", log_info.output)
+        self.assertEqual(status, 2)
+
 
 class TestMappingPopulate(unit.SQLDriverOverrides, unit.TestCase):
 
@@ -772,7 +773,7 @@ class CliDomainConfigUploadNothing(unit.BaseTestCase):
         # setup a test database.
         def fake_load_backends(self):
             self.resource_manager = mock.Mock()
-        self.useFixture(mockpatch.PatchObject(
+        self.useFixture(fixtures.MockPatchObject(
             cli.DomainConfigUploadFiles, 'load_backends', fake_load_backends))
 
         tempdir = self.useFixture(fixtures.TempDir())
@@ -1173,11 +1174,8 @@ class SecurityComplianceDoctorTests(unit.TestCase):
             symptom_minimum_password_age_greater_than_expires_days())
 
     def test_minimum_password_age_and_password_expires_days_deactivated(self):
-        # No Symptom Detected: Both values are deactivated to 0
-        self.config_fixture.config(group='security_compliance',
-                                   minimum_password_age=0)
-        self.config_fixture.config(group='security_compliance',
-                                   password_expires_days=0)
+        # No Symptom Detected: when minimum_password_age's default value is 0
+        # and password_expires_days' default value is None
         self.assertFalse(
             security_compliance.
             symptom_minimum_password_age_greater_than_expires_days())
