@@ -16,12 +16,13 @@ import copy
 import hashlib
 import uuid
 
-from oslo_config import cfg
+import fixtures
 from six.moves import http_client
 import webtest
 
 from keystone.common import authorization
 from keystone.common import tokenless_auth
+import keystone.conf
 from keystone import exception
 from keystone.federation import constants as federation_constants
 from keystone import middleware
@@ -30,7 +31,7 @@ from keystone.tests.unit import mapping_fixtures
 from keystone.tests.unit import test_backend_sql
 
 
-CONF = cfg.CONF
+CONF = keystone.conf.CONF
 
 
 class MiddlewareRequestTestBase(unit.TestCase):
@@ -72,10 +73,10 @@ class MiddlewareRequestTestBase(unit.TestCase):
 
             _called = False
 
-            def process_request(i_self, *i_args, **i_kwargs):
+            def fill_context(i_self, *i_args, **i_kwargs):
                 # i_ to distinguish it from and not clobber the outer vars
                 e = self.assertRaises(exc,
-                                      super(_Failing, i_self).process_request,
+                                      super(_Failing, i_self).fill_context,
                                       *i_args, **i_kwargs)
                 i_self._called = True
                 raise e
@@ -110,26 +111,6 @@ class TokenAuthMiddlewareTest(MiddlewareRequestTestBase):
         self.assertEqual('MAGIC', context['token_id'])
 
 
-class AdminTokenAuthMiddlewareTest(MiddlewareRequestTestBase):
-
-    MIDDLEWARE_CLASS = middleware.AdminTokenAuthMiddleware
-
-    def config_overrides(self):
-        super(AdminTokenAuthMiddlewareTest, self).config_overrides()
-        self.config_fixture.config(
-            admin_token='ADMIN')
-
-    def test_request_admin(self):
-        headers = {middleware.AUTH_TOKEN_HEADER: 'ADMIN'}
-        req = self._do_middleware_request(headers=headers)
-        self.assertTrue(req.environ[middleware.CONTEXT_ENV]['is_admin'])
-
-    def test_request_non_admin(self):
-        headers = {middleware.AUTH_TOKEN_HEADER: 'NOT-ADMIN'}
-        req = self._do_middleware_request(headers=headers)
-        self.assertFalse(req.environ[middleware.CONTEXT_ENV]['is_admin'])
-
-
 class JsonBodyMiddlewareTest(MiddlewareRequestTestBase):
 
     MIDDLEWARE_CLASS = middleware.JsonBodyMiddleware
@@ -300,33 +281,6 @@ class AuthContextMiddlewareTest(test_backend_sql.SqlTests,
         else:
             self.assertEqual(self.user['id'], context['user_id'])
 
-    def _create_context(self, request, mapping_ref=None,
-                        exception_expected=False):
-        """Builds the auth context from the given arguments.
-
-        auth context will be returned from the AuthContextMiddleware based on
-        what is being passed in the given request and what mapping is being
-        setup in the backend DB.
-
-        :param request: HTTP request
-        :param mapping_ref: A mapping in JSON structure will be setup in the
-            backend DB for mapping a user or a group.
-        :param exception_expected: Sets to True when an exception is expected
-            to raised based on the given arguments.
-        :returns: context an auth context contains user and role information
-        :rtype: dict
-        """
-        if mapping_ref:
-            self._load_mapping_rules(mapping_ref)
-
-        if not exception_expected:
-            (middleware.AuthContextMiddleware('Tokenless_auth_test').
-                process_request(request))
-            context = request.environ.get(authorization.AUTH_CONTEXT_ENV)
-        else:
-            context = middleware.AuthContextMiddleware('Tokenless_auth_test')
-        return context
-
     def test_context_already_exists(self):
         stub_value = uuid.uuid4().hex
         env = {authorization.AUTH_CONTEXT_ENV: stub_value}
@@ -759,6 +713,22 @@ class AuthContextMiddlewareTest(test_backend_sql.SqlTests,
                         'request environment.' %
                         CONF.tokenless_auth.issuer_attribute)
         # Check the content of the exception message as well
-        self.assertRaisesRegexp(exception.TokenlessAuthConfigError,
-                                expected_msg,
-                                auth._build_idp_id)
+        self.assertRaisesRegex(exception.TokenlessAuthConfigError,
+                               expected_msg,
+                               auth._build_idp_id)
+
+    def test_admin_token_context(self):
+        self.config_fixture.config(admin_token='ADMIN')
+        log_fix = self.useFixture(fixtures.FakeLogger())
+        headers = {middleware.AUTH_TOKEN_HEADER: 'ADMIN'}
+        req = self._do_middleware_request(headers=headers)
+        self.assertTrue(req.environ[middleware.CONTEXT_ENV]['is_admin'])
+        self.assertNotIn('Invalid user token', log_fix.output)
+
+    def test_request_non_admin(self):
+        self.config_fixture.config(
+            admin_token='ADMIN')
+        log_fix = self.useFixture(fixtures.FakeLogger())
+        headers = {middleware.AUTH_TOKEN_HEADER: 'NOT-ADMIN'}
+        self._do_middleware_request(headers=headers)
+        self.assertIn('Invalid user token', log_fix.output)
