@@ -13,14 +13,17 @@
 # under the License.
 
 import copy
+import datetime
 import random
 import uuid
 
+import freezegun
 import mock
 from oslo_serialization import jsonutils
 from pycadf import cadftaxonomy
 from six.moves import http_client
 from six.moves import urllib
+from six.moves.urllib import parse as urlparse
 
 import keystone.conf
 from keystone import exception
@@ -60,19 +63,21 @@ class OAuth1Tests(test_v3.RestfulTestCase):
             body={'consumer': ref})
         return resp.result['consumer']
 
-    def _create_request_token(self, consumer, project_id):
+    def _create_request_token(self, consumer, project_id, base_url=None):
         endpoint = '/OS-OAUTH1/request_token'
         client = oauth1.Client(consumer['key'],
                                client_secret=consumer['secret'],
                                signature_method=oauth1.SIG_HMAC,
                                callback_uri="oob")
         headers = {'requested_project_id': project_id}
-        url, headers, body = client.sign(self.base_url + endpoint,
+        if not base_url:
+            base_url = self.base_url
+        url, headers, body = client.sign(base_url + endpoint,
                                          http_method='POST',
                                          headers=headers)
         return endpoint, headers
 
-    def _create_access_token(self, consumer, token):
+    def _create_access_token(self, consumer, token, base_url=None):
         endpoint = '/OS-OAUTH1/access_token'
         client = oauth1.Client(consumer['key'],
                                client_secret=consumer['secret'],
@@ -80,7 +85,9 @@ class OAuth1Tests(test_v3.RestfulTestCase):
                                resource_owner_secret=token.secret,
                                signature_method=oauth1.SIG_HMAC,
                                verifier=token.verifier)
-        url, headers, body = client.sign(self.base_url + endpoint,
+        if not base_url:
+            base_url = self.base_url
+        url, headers, body = client.sign(base_url + endpoint,
                                          http_method='POST')
         headers.update({'Content-Type': 'application/json'})
         return endpoint, headers
@@ -150,16 +157,19 @@ class ConsumerCRUDTests(OAuth1Tests):
         resp = self.delete(self.CONSUMER_URL + '/%s' % consumer_id)
         self.assertResponseStatus(resp, http_client.NO_CONTENT)
 
-    def test_consumer_get(self):
+    def test_consumer_get_head(self):
         consumer = self._create_single_consumer()
         consumer_id = consumer['id']
-        resp = self.get(self.CONSUMER_URL + '/%s' % consumer_id)
+        url = self.CONSUMER_URL + '/%s' % consumer_id
+        resp = self.get(url)
         self_url = ['http://localhost/v3', self.CONSUMER_URL,
                     '/', consumer_id]
         self_url = ''.join(self_url)
         self.assertEqual(self_url, resp.result['consumer']['links']['self'])
         self.assertEqual(consumer_id, resp.result['consumer']['id'])
 
+        self.head(url, expected_status=http_client.OK)
+
     def test_consumer_list(self):
         self._consumer_create()
         resp = self.get(self.CONSUMER_URL)
@@ -170,6 +180,8 @@ class ConsumerCRUDTests(OAuth1Tests):
         self.assertEqual(self_url, resp.result['links']['self'])
         self.assertValidListLinks(resp.result['links'])
 
+        self.head(self.CONSUMER_URL, expected_status=http_client.OK)
+
     def test_consumer_update(self):
         consumer = self._create_single_consumer()
         original_id = consumer['id']
@@ -243,9 +255,12 @@ class ConsumerCRUDTests(OAuth1Tests):
         self.assertIsNotNone(consumer['secret'])
 
     def test_consumer_get_bad_id(self):
-        self.get(self.CONSUMER_URL + '/%(consumer_id)s'
-                 % {'consumer_id': uuid.uuid4().hex},
-                 expected_status=http_client.NOT_FOUND)
+        url = (
+            self.CONSUMER_URL + '/%(consumer_id)s' %
+            {'consumer_id': uuid.uuid4().hex}
+        )
+        self.get(url, expected_status=http_client.NOT_FOUND)
+        self.head(url, expected_status=http_client.NOT_FOUND)
 
 
 class OAuthFlowTests(OAuth1Tests):
@@ -303,12 +318,17 @@ class AccessTokenCRUDTests(OAuthFlowTests):
                     expected_status=http_client.NOT_FOUND)
 
     def test_list_no_access_tokens(self):
-        resp = self.get('/users/%(user_id)s/OS-OAUTH1/access_tokens'
-                        % {'user_id': self.user_id})
+        url = (
+            '/users/%(user_id)s/OS-OAUTH1/access_tokens'
+            % {'user_id': self.user_id}
+        )
+        resp = self.get(url)
         entities = resp.result['access_tokens']
         self.assertEqual([], entities)
         self.assertValidListLinks(resp.result['links'])
 
+        self.head(url, expected_status=http_client.OK)
+
     def test_get_single_access_token(self):
         self.test_oauth_flow()
         access_token_key_string = self.access_token.key.decode()
@@ -323,21 +343,31 @@ class AccessTokenCRUDTests(OAuthFlowTests):
         self.assertEqual(self.consumer['key'], entity['consumer_id'])
         self.assertEqual('http://localhost/v3' + url, entity['links']['self'])
 
+        self.head(url, expected_status=http_client.OK)
+
     def test_get_access_token_dne(self):
-        self.get('/users/%(user_id)s/OS-OAUTH1/access_tokens/%(key)s'
-                 % {'user_id': self.user_id,
-                    'key': uuid.uuid4().hex},
-                 expected_status=http_client.NOT_FOUND)
+        url = (
+            '/users/%(user_id)s/OS-OAUTH1/access_tokens/%(key)s'
+            % {'user_id': self.user_id,
+               'key': uuid.uuid4().hex}
+        )
+        self.get(url, expected_status=http_client.NOT_FOUND)
+        self.head(url, expected_status=http_client.NOT_FOUND)
 
     def test_list_all_roles_in_access_token(self):
         self.test_oauth_flow()
-        resp = self.get('/users/%(id)s/OS-OAUTH1/access_tokens/%(key)s/roles'
-                        % {'id': self.user_id,
-                           'key': self.access_token.key.decode()})
+        url = (
+            '/users/%(id)s/OS-OAUTH1/access_tokens/%(key)s/roles'
+            % {'id': self.user_id,
+               'key': self.access_token.key.decode()}
+        )
+        resp = self.get(url)
         entities = resp.result['roles']
         self.assertTrue(entities)
         self.assertValidListLinks(resp.result['links'])
 
+        self.head(url, expected_status=http_client.OK)
+
     def test_get_role_in_access_token(self):
         self.test_oauth_flow()
 
@@ -349,6 +379,8 @@ class AccessTokenCRUDTests(OAuthFlowTests):
         entity = resp.result['role']
         self.assertEqual(self.role_id, entity['id'])
 
+        self.head(url, expected_status=http_client.OK)
+
     def test_get_role_in_access_token_dne(self):
         self.test_oauth_flow()
 
@@ -357,12 +389,17 @@ class AccessTokenCRUDTests(OAuthFlowTests):
                % {'id': self.user_id, 'key': access_token_key,
                   'role': uuid.uuid4().hex})
         self.get(url, expected_status=http_client.NOT_FOUND)
+        self.head(url, expected_status=http_client.NOT_FOUND)
 
     def test_list_and_delete_access_tokens(self):
         self.test_oauth_flow()
         # List access_tokens should be > 0
-        resp = self.get('/users/%(user_id)s/OS-OAUTH1/access_tokens'
-                        % {'user_id': self.user_id})
+        url = (
+            '/users/%(user_id)s/OS-OAUTH1/access_tokens'
+            % {'user_id': self.user_id}
+        )
+        resp = self.get(url)
+        self.head(url, expected_status=http_client.OK)
         entities = resp.result['access_tokens']
         self.assertTrue(entities)
         self.assertValidListLinks(resp.result['links'])
@@ -375,8 +412,8 @@ class AccessTokenCRUDTests(OAuthFlowTests):
         self.assertResponseStatus(resp, http_client.NO_CONTENT)
 
         # List access_token should be 0
-        resp = self.get('/users/%(user_id)s/OS-OAUTH1/access_tokens'
-                        % {'user_id': self.user_id})
+        resp = self.get(url)
+        self.head(url, expected_status=http_client.OK)
         entities = resp.result['access_tokens']
         self.assertEqual([], entities)
         self.assertValidListLinks(resp.result['links'])
@@ -630,6 +667,13 @@ class UUIDAuthTokenTests(AuthTokenTests, OAuthFlowTests):
 
 class MaliciousOAuth1Tests(OAuth1Tests):
 
+    def _switch_baseurl_scheme(self):
+        """Switch the base url scheme."""
+        base_url_list = list(urlparse.urlparse(self.base_url))
+        base_url_list[0] = 'https' if base_url_list[0] == 'http' else 'http'
+        bad_url = urlparse.urlunparse(base_url_list)
+        return bad_url
+
     def test_bad_consumer_secret(self):
         consumer = self._create_single_consumer()
         consumer_id = consumer['id']
@@ -638,6 +682,28 @@ class MaliciousOAuth1Tests(OAuth1Tests):
         self.post(url, headers=headers,
                   expected_status=http_client.UNAUTHORIZED)
 
+    def test_bad_request_url(self):
+        consumer = self._create_single_consumer()
+        consumer_id = consumer['id']
+        consumer_secret = consumer['secret']
+        consumer = {'key': consumer_id, 'secret': consumer_secret}
+        bad_base_url = 'http://localhost/identity_admin/v3'
+        url, headers = self._create_request_token(consumer, self.project_id,
+                                                  base_url=bad_base_url)
+        self.post(url, headers=headers,
+                  expected_status=http_client.UNAUTHORIZED)
+
+    def test_bad_request_url_scheme(self):
+        consumer = self._create_single_consumer()
+        consumer_id = consumer['id']
+        consumer_secret = consumer['secret']
+        consumer = {'key': consumer_id, 'secret': consumer_secret}
+        bad_url_scheme = self._switch_baseurl_scheme()
+        url, headers = self._create_request_token(consumer, self.project_id,
+                                                  base_url=bad_url_scheme)
+        self.post(url, headers=headers,
+                  expected_status=http_client.UNAUTHORIZED)
+
     def test_bad_request_token_key(self):
         consumer = self._create_single_consumer()
         consumer_id = consumer['id']
@@ -720,7 +786,29 @@ class MaliciousOAuth1Tests(OAuth1Tests):
         verifier = resp.result['token']['oauth_verifier']
         request_token.set_verifier(verifier)
 
-        # 1. Invalid signature.
+        # 1. Invalid base url.
+        # Update the base url, so it will fail to validate the signature.
+        base_url = 'http://localhost/identity_admin/v3'
+        url, headers = self._create_access_token(consumer, request_token,
+                                                 base_url=base_url)
+        resp = self.post(url, headers=headers,
+                         expected_status=http_client.UNAUTHORIZED)
+        resp_data = jsonutils.loads(resp.body)
+        self.assertIn('Invalid signature',
+                      resp_data.get('error', {}).get('message'))
+
+        # 2. Invalid base url scheme.
+        # Update the base url scheme, so it will fail to validate signature.
+        bad_url_scheme = self._switch_baseurl_scheme()
+        url, headers = self._create_access_token(consumer, request_token,
+                                                 base_url=bad_url_scheme)
+        resp = self.post(url, headers=headers,
+                         expected_status=http_client.UNAUTHORIZED)
+        resp_data = jsonutils.loads(resp.body)
+        self.assertIn('Invalid signature',
+                      resp_data.get('error', {}).get('message'))
+
+        # 3. Invalid signature.
         # Update the secret, so it will fail to validate the signature.
         consumer.update({'secret': uuid.uuid4().hex})
         url, headers = self._create_access_token(consumer, request_token)
@@ -730,7 +818,7 @@ class MaliciousOAuth1Tests(OAuth1Tests):
         self.assertIn('Invalid signature',
                       resp_data.get('error', {}).get('message'))
 
-        # 2. Invalid verifier.
+        # 4. Invalid verifier.
         # Even though the verifier is well formatted, it is not verifier
         # that is stored in the backend, this is different with the testcase
         # above `test_bad_verifier` where it test that `verifier` is not
@@ -745,7 +833,7 @@ class MaliciousOAuth1Tests(OAuth1Tests):
         self.assertIn('Provided verifier',
                       resp_data.get('error', {}).get('message'))
 
-        # 3. The provided consumer does not exist.
+        # 5. The provided consumer does not exist.
         consumer.update({'key': uuid.uuid4().hex})
         url, headers = self._create_access_token(consumer, request_token)
         resp = self.post(url, headers=headers,
@@ -754,7 +842,7 @@ class MaliciousOAuth1Tests(OAuth1Tests):
         self.assertIn('Provided consumer does not exist',
                       resp_data.get('error', {}).get('message'))
 
-        # 4. The consumer key provided does not match stored consumer key.
+        # 6. The consumer key provided does not match stored consumer key.
         consumer2 = self._create_single_consumer()
         consumer.update({'key': consumer2['id']})
         url, headers = self._create_access_token(consumer, request_token)
@@ -839,69 +927,78 @@ class MaliciousOAuth1Tests(OAuth1Tests):
                       resp_data['error']['message'])
 
     def test_expired_authorizing_request_token(self):
-        self.config_fixture.config(group='oauth1', request_token_duration=-1)
+        with freezegun.freeze_time(datetime.datetime.utcnow()) as frozen_time:
+            self.config_fixture.config(group='oauth1',
+                                       request_token_duration=1)
 
-        consumer = self._create_single_consumer()
-        consumer_id = consumer['id']
-        consumer_secret = consumer['secret']
-        self.consumer = {'key': consumer_id, 'secret': consumer_secret}
-        self.assertIsNotNone(self.consumer['key'])
+            consumer = self._create_single_consumer()
+            consumer_id = consumer['id']
+            consumer_secret = consumer['secret']
+            self.consumer = {'key': consumer_id, 'secret': consumer_secret}
+            self.assertIsNotNone(self.consumer['key'])
 
-        url, headers = self._create_request_token(self.consumer,
-                                                  self.project_id)
-        content = self.post(
-            url, headers=headers,
-            response_content_type='application/x-www-form-urlencoded')
-        credentials = _urllib_parse_qs_text_keys(content.result)
-        request_key = credentials['oauth_token'][0]
-        request_secret = credentials['oauth_token_secret'][0]
-        self.request_token = oauth1.Token(request_key, request_secret)
-        self.assertIsNotNone(self.request_token.key)
+            url, headers = self._create_request_token(self.consumer,
+                                                      self.project_id)
+            content = self.post(
+                url, headers=headers,
+                response_content_type='application/x-www-form-urlencoded')
+            credentials = _urllib_parse_qs_text_keys(content.result)
+            request_key = credentials['oauth_token'][0]
+            request_secret = credentials['oauth_token_secret'][0]
+            self.request_token = oauth1.Token(request_key, request_secret)
+            self.assertIsNotNone(self.request_token.key)
 
-        url = self._authorize_request_token(request_key)
-        body = {'roles': [{'id': self.role_id}]}
-        self.put(url, body=body, expected_status=http_client.UNAUTHORIZED)
+            url = self._authorize_request_token(request_key)
+            body = {'roles': [{'id': self.role_id}]}
+            frozen_time.tick(delta=datetime.timedelta(
+                seconds=CONF.oauth1.request_token_duration + 1))
+            self.put(url, body=body, expected_status=http_client.UNAUTHORIZED)
 
     def test_expired_creating_keystone_token(self):
-        self.config_fixture.config(group='oauth1', access_token_duration=-1)
-        consumer = self._create_single_consumer()
-        consumer_id = consumer['id']
-        consumer_secret = consumer['secret']
-        self.consumer = {'key': consumer_id, 'secret': consumer_secret}
-        self.assertIsNotNone(self.consumer['key'])
+        with freezegun.freeze_time(datetime.datetime.utcnow()) as frozen_time:
+            self.config_fixture.config(group='oauth1',
+                                       access_token_duration=1)
 
-        url, headers = self._create_request_token(self.consumer,
-                                                  self.project_id)
-        content = self.post(
-            url, headers=headers,
-            response_content_type='application/x-www-form-urlencoded')
-        credentials = _urllib_parse_qs_text_keys(content.result)
-        request_key = credentials['oauth_token'][0]
-        request_secret = credentials['oauth_token_secret'][0]
-        self.request_token = oauth1.Token(request_key, request_secret)
-        self.assertIsNotNone(self.request_token.key)
+            consumer = self._create_single_consumer()
+            consumer_id = consumer['id']
+            consumer_secret = consumer['secret']
+            self.consumer = {'key': consumer_id, 'secret': consumer_secret}
+            self.assertIsNotNone(self.consumer['key'])
 
-        url = self._authorize_request_token(request_key)
-        body = {'roles': [{'id': self.role_id}]}
-        resp = self.put(url, body=body, expected_status=http_client.OK)
-        self.verifier = resp.result['token']['oauth_verifier']
+            url, headers = self._create_request_token(self.consumer,
+                                                      self.project_id)
+            content = self.post(
+                url, headers=headers,
+                response_content_type='application/x-www-form-urlencoded')
+            credentials = _urllib_parse_qs_text_keys(content.result)
+            request_key = credentials['oauth_token'][0]
+            request_secret = credentials['oauth_token_secret'][0]
+            self.request_token = oauth1.Token(request_key, request_secret)
+            self.assertIsNotNone(self.request_token.key)
 
-        self.request_token.set_verifier(self.verifier)
-        url, headers = self._create_access_token(self.consumer,
-                                                 self.request_token)
-        content = self.post(
-            url, headers=headers,
-            response_content_type='application/x-www-form-urlencoded')
-        credentials = _urllib_parse_qs_text_keys(content.result)
-        access_key = credentials['oauth_token'][0]
-        access_secret = credentials['oauth_token_secret'][0]
-        self.access_token = oauth1.Token(access_key, access_secret)
-        self.assertIsNotNone(self.access_token.key)
+            url = self._authorize_request_token(request_key)
+            body = {'roles': [{'id': self.role_id}]}
+            resp = self.put(url, body=body, expected_status=http_client.OK)
+            self.verifier = resp.result['token']['oauth_verifier']
 
-        url, headers, body = self._get_oauth_token(self.consumer,
-                                                   self.access_token)
-        self.post(url, headers=headers, body=body,
-                  expected_status=http_client.UNAUTHORIZED)
+            self.request_token.set_verifier(self.verifier)
+            url, headers = self._create_access_token(self.consumer,
+                                                     self.request_token)
+            content = self.post(
+                url, headers=headers,
+                response_content_type='application/x-www-form-urlencoded')
+            credentials = _urllib_parse_qs_text_keys(content.result)
+            access_key = credentials['oauth_token'][0]
+            access_secret = credentials['oauth_token_secret'][0]
+            self.access_token = oauth1.Token(access_key, access_secret)
+            self.assertIsNotNone(self.access_token.key)
+
+            url, headers, body = self._get_oauth_token(self.consumer,
+                                                       self.access_token)
+            frozen_time.tick(delta=datetime.timedelta(
+                seconds=CONF.oauth1.access_token_duration + 1))
+            self.post(url, headers=headers, body=body,
+                      expected_status=http_client.UNAUTHORIZED)
 
     def test_missing_oauth_headers(self):
         endpoint = '/OS-OAUTH1/request_token'
